import reveallib
import reveallib64
from utils import *
from intervaltree import IntervalTree
import networkx as nx
import uuid

def chain_cmd(args):
    fastas=args.fastas
    idx=reveallib.index()
    minn=args.minn
    
    tree=IntervalTree()
    
    for fasta in fastas:
        sample=os.path.basename(fasta)
        idx.addsample(sample)
        for i,t in enumerate(fasta_reader(fasta)):
            name,seq=t
            f,t=idx.addsequence(seq)
            tree[f:t]=sample
            if i==1:
                logging.error("Can't handle multi-fasta input. Use single fasta file per sequence.")
                sys.exit(1)
    
    idx.construct()
    
    G=nx.DiGraph()
    G.graph['paths']=idx.samples
    G.graph['path2id']=dict()
    G.graph['id2path']=dict()
    G.graph['startnodes']=[]
    G.graph['endnodes']=[]

    for sid,sample in enumerate(G.graph['paths']):
        G.graph['path2id'][sample]=sid
        G.graph['id2path'][sid]=sample
    
    k=len(idx.samples)
    
    T=idx.T
    
    istart=tuple([-1]+[sep for sep in idx.nsep]) #no matches possible at these loci
    iend=tuple([sep for sep in idx.nsep]+[idx.n-1]) #loci of sentinels, also no matches possible
    startcoords=tuple([0]+[sep+1 for sep in idx.nsep])
    G.add_node(istart,l=0)
    G.add_node(iend,l=0)
    G.add_edge(istart,iend)
    
    G.graph['startnodes'].append(istart)
    G.graph['endnodes'].append(iend)
    
    idc=range(idx.nsamples)

    stack=[(idx,idc,istart,iend,startcoords,0,False)]
    
    while len(stack)!=0:
        idx,idc,p1,p2,startcoords,depth,keepedge=stack.pop()
        subg,pp1,pp2,nodepath=chain(idx,startcoords,args.minlength,depth,args.maxmums,recurse=args.recurse,uniq=True,gcmodel=args.gcmodel,wpen=args.wpen,wscore=args.wscore)

        if len(nodepath)==2: #no more chain, output variant sequence
            localstart=tuple([-1]+[sep for sep in idx.nsep])
            localend=tuple([sep-1 for sep in idx.nsep]+[idx.n-2])
            lengths=tuple([e-s for s,e in zip(localstart,localend)])
            outputVariantNodes(G,T,p1,p2,startcoords,lengths)
            if not keepedge:
                G.remove_edge(p1,p2)
            continue
         
        #replace the edge (start,end) in G with the chain in subg
        insertSubgraph(G,p1,p2,subg,pp1,pp2,keepedge)
        
        coordpath=list(nodepath)
        coordpath[0]=tuple([d+1 for d in nodepath[0]])
        nodepath[0]=p1
        nodepath[-1]=p2

        fromcoord=coordpath[0]
        fromnode=nodepath[0]
        l=0
        
        #for every edge in subg construct idx and add to stack
        for node,pos in zip(nodepath[1:],coordpath[1:]):
            seq=[]
            idc_=[]
            keepedge=False

            for i in idc:
                f=fromcoord[i]
                t=pos[i]
                assert(f>=0)
                assert(t>=0)
                if f+l<t:
                    seq.append(T[f+l:t])
                    idc_.append(i)
                elif f+l==t:
                    keepedge=True
                else:
                    print "Error overlapping matches",f,l,t
                    sys.exit(1)
            
            if len(seq)>=minn and args.recurse==True:
                idx=reveallib.index()
                for i,s in enumerate(seq):
                    assert('$' not in s)
                    idx.addsample(str(i))
                    idx.addsequence(s)
                idx.construct()
                
                newoffsets=tuple([fromcoord[i]+l for i in idc_])
                idc_=range(len(newoffsets))
                stack.append((idx, idc_, fromnode, node, newoffsets, depth+1, keepedge))
            else:
                varnodes=[fromcoord[i]+l for i in idc_]
                lengths=[pos[i]-(fromcoord[i]+l) for i in idc_]
                outputVariantNodes(G,T,fromnode,node,varnodes,lengths)
                if not keepedge:
                    G.remove_edge(fromnode,node)
            
            fromcoord=pos
            fromnode=node
            
            if node!=nodepath[-1]:
                l=subg.node[node]['l']
    
    G.remove_node(istart)
    G.remove_node(iend)
    
    tot=0
    totn=0
    for node,data in G.nodes(data=True):
        G.node[node]['offsets']=dict()
        
        if isinstance(node,tuple):
            G.node[node]['seq']=T[node[0]:node[0]+data['l']]
            for c in node:
                intv=list(tree[c])[0]
                G.node[node]['offsets'][G.graph['path2id'][intv[2]]]=c-intv[0]
        else:
            if 'l' in data:
                G.node[node]['seq']=T[node:node+data['l']]
            intv=list(tree[node])[0]
            G.node[node]['offsets'][G.graph['path2id'][intv[2]]]=node-intv[0]
        
        if 'aligned' in data:
            if data['aligned']==1:
                tot+=data['l']
                totn+=1

    print "Aligned",tot,"bases in",totn,"nodes. Nodes total:",G.number_of_nodes(),"Edges total:",G.number_of_edges()
    
    if args.mumplot:
        plotgraph(G, G.graph['paths'][0], G.graph['paths'][1], interactive=args.interactive)
    
    if args.output==None:
        pref=[]
        for f in args.fastas:
            bn=os.path.basename(f)
            if '.' in bn:
                pref.append(bn[:bn.find('.')])
            else:
                pref.append(bn)
        args.output="_".join(pref)
    
    #add paths annotation to edges
    for sample in G.graph['paths']:
        sid=G.graph['path2id'][sample]
        sg=[]
        for node,data in G.nodes(data=True):
            if sid in data['offsets']:
                sg.append(node)
        subgraph=G.subgraph(sg)
        topsort=list(nx.topological_sort(subgraph))
        pnode=topsort[0]
        for node in topsort[1:]:
            if 'paths' in G[pnode][node]:
                G[pnode][node]['paths'].add(sid)
            else:
                G[pnode][node]['paths']={sid}
            pnode=node

    write_gfa(G,T,nometa=args.nometa,outputfile=args.output+'.gfa')

def outputVariantNodes(G,T,source,sink,varnodes,lengths,merge=True):
    if merge:
        seq=[]
        uvarseq=dict()
        gaps=[]
        for n,l in zip(varnodes,lengths):
            s=T[n:n+l]
            if 'N' not in s: #dont merge gaps, biases statistics
                if s in uvarseq:
                    uvarseq[s]+=[n]
                else:
                    uvarseq[s]=[n]
            else:
                gaps.append((n,l))
        
        for uv in uvarseq:
            G.add_node(tuple(uvarseq[uv]),l=len(uv),aligned=1 if len(uvarseq[uv])>1 else 0)
            G.add_edge(source,tuple(uvarseq[uv]))
            G.add_edge(tuple(uvarseq[uv]),sink)
        
        for gap,l in gaps:
            G.add_node(tuple([gap]),l=l,aligned=0)
            G.add_edge(source,tuple([gap]))
            G.add_edge(tuple([gap]),sink)
    else:
        for v,l in zip(varnodes,lengths):
            G.add_node(v,l=l)
            G.add_edge(source,v)
            G.add_edge(v,sink)

def chain(idx,offsets,minlength,depth,maxmums,recurse=True,uniq=True,gcmodel="sumofpairs",wpen=1,wscore=1):
    k=idx.nsamples
    
    if k>2:
        logging.debug("Extracting mmums of length %d."%minlength)
        mums=idx.getmultimums(minlength=minlength,minn=k)
        logging.debug("Extracted %d mums."%len(mums))
    else:
        logging.debug("Extracting mums of length %d."%minlength)
        mums=idx.getmums(minlength)
        logging.debug("Extracted %d mums."%len(mums))
    
    points=[]
    G=nx.DiGraph()
    localoffsets=tuple([0]+[sep+1 for sep in idx.nsep])
    localstart=tuple([-1]+[sep for sep in idx.nsep])
    localend=tuple([sep-1 for sep in idx.nsep]+[idx.n-2])
    lengths=tuple([e-s for s,e in zip(localstart,localend)])

    p1=tuple([o-1 for o in offsets])
    p2=tuple([o+l for o,l in zip(offsets,lengths)])
    
    mums=[m for m in mums if m[1]==k] #filter only mums that occur in all genomes
    
    if len(mums)>maxmums and maxmums!=0:
        logging.info("Capping the %d anchors that were detected, taking the maxmums=%d longest for chaining."%(len(mums),maxmums))
        mums=sorted(mums,key=lambda m: m[0])[-maxmums:] #take top n longest mums
    elif len(mums)==0:
        return G,p1,p2,[p1,p2]
    else:
        #print "Found %d anchors."%len(mums)
        mums=sorted(mums,key=lambda m: m[0])
    
    logging.debug("Computing chain for %d mums."%len(mums))
    #add all nodes to the graph
    for mum in mums:
        # point=sorted(mum[2])
        # point=sorted(mum[2].values())
        point=sorted([sp for gid,sp in mum[2]])
        for i,p in enumerate(point):
            point[i]=offsets[i]+(point[i]-localoffsets[i]) #map positions back to toplevel T index
        point=tuple(point)
        points.append(point)
        G.add_node(point,l=mum[0])
    
    G.add_node(p1,s=0,l=0,score=0)
    G.add_node(p2,s=0,l=0,score=0)
    
    points.append(p2)
    points=sorted(points,key=lambda p:p[0]) #sort points by first dimension
    
    #build the k-dimensional tree for fast k-dimensional range queries
    tree=kdtree(points,k)
    
    #add edges to graph
    for t in points:
        bestpoint=p1
        bestpenalty=gapcost(p1,t,model=gcmodel)
        bestscore=-1*(wpen*bestpenalty)
        for v in range_search(tree,p1,t):
            if v==t:
                continue
            l=G.node[v]['l']
            for i,d in enumerate(v): #no overlapping mums
                if d+l>t[i]:
                    break
            else:
                penalty=gapcost(v,t,model=gcmodel)
                score=G.node[v]['score']+(wscore*(l*((len(v)*(len(v)-1))/2)))-(wpen*penalty)
                if score>bestscore:
                    bestscore=score
                    bestpoint=v
                    bestpenalty=penalty

        G.node[t]['score']=bestscore
        G.add_edge(bestpoint,t,p=bestpenalty)
    
    logging.debug("backtrack")
    bestpath=[]
    #backtrack the optimal path
    v=p2
    while v!=p1:
        bestpath.append(v)
        G.node[v]['aligned']=1
        for v in G.predecessors(v):
            break

    bestpath.append(p1)
    
    logging.debug("remove nodes")
    #remove nodes that aren't part of the optimal path
    delete=[]
    bestpaths=set(bestpath)
    for node in G.nodes():
        if node not in bestpaths:
            delete.append(node)
    #remove nodes that are not contained in the bestpath
    G.remove_nodes_from(delete)
    logging.debug("done")

    return G,p1,p2,bestpath[::-1]

def insertSubgraph(G,start,end,subg,sstart,send,keepedge):
    upref=uuid.uuid4().hex
    mapping={ sstart : upref+str(sstart), send : upref+str(send) }
    nx.relabel_nodes(subg,mapping,copy=False) #relabel the start and end node in the subgraph to prevent overlap in id space
    
    for node in subg.nodes():
        if node in G:
            print "node already exists!",node
        assert(node not in G)
    
    G.add_nodes_from(subg.nodes(data=True))
    G.add_edges_from(subg.edges(data=True))

    for nei in G.successors(upref+str(sstart)):
        G.add_edge(start,nei)
    
    for nei in G.predecessors(upref+str(send)):
        G.add_edge(nei,end)
    
    if not keepedge:
        G.remove_edge(start,end)

    G.remove_node(upref+str(send))
    G.remove_node(upref+str(sstart))




import networkx as nx
from utils import *
import sys

def split_cmd(args):
    if len(args.gfa)!=1:
        logging.fatal("Specify 1 gfa file.")
        return
    
    if not args.gfa[0].endswith('.gfa'):
        logging.fatal("Use .gfa as extension of the gfa file.")
        return
    
    if args.nocycles:
        G=nx.DiGraph()
    else:
        G=nx.MultiDiGraph()

    split(G,args.gfa[0])

def split(G,gfafile):

    logging.debug("Reading graph...")
    read_gfa(gfafile,None,"",G)
    logging.debug("Done.")

    for i,sg in enumerate(nx.weakly_connected_component_subgraphs(G)):
        
        sgpaths=[]

        sids=set()
        for node in sg.nodes():
            if type(node)!=str:
                for sid in sg.node[node]['offsets']:
                    sids.add(sid)
        
        for sid in sids:
            sgpaths.append(sg.graph['id2path'][sid])

        sg.graph['paths']=sgpaths
        sg.graph['id2path']={sid:sg.graph['id2path'][sid] for sid in sids}
        sg.graph['path2id']={path:sg.graph['path2id'][path] for path in sgpaths}

        name="_".join([p for p in sorted(sgpaths) if not p.startswith("*")]).replace("|","_").replace(" ","_")[:200]

        logging.info("Write component (%d, size=%d) to: %s"%(i,len(sg.nodes()),name))
        write_gfa(sg,None,outputfile="%s.gfa"%name)


from utils import *
from extract import extract,extract_path
from rem import align,prune_nodes
from random import shuffle
import bubbles
import schemes
import multiprocessing as mp
import signal
import probconslib
import math
import os
import time

def refine_bubble_cmd(args):
    if len(args.graph)<1:
        logging.fatal("Specify a gfa file for which bubbles should be realigned.")
        return
    
    # G=nx.MultiDiGraph() #TODO: make sure that refine can handle structural variant edges, so make sure we use a MultiDiGraph here!
    G=nx.DiGraph()

    logging.info("Reading graph...")
    read_gfa(args.graph[0],None,"",G)
    logging.info("Done.")

    logging.info("Paths through the graph: %s"%G.graph['paths'])

    if (args.source==None and args.sink==None) and (args.all or args.complex or args.simple):
        G=refine_all(G, **vars(args))
    else:
        if args.source==None or args.sink==None:
            logging.error("Specify source sink pair, or one of the --all --simple --complex flags.")
            sys.exit(1)

        args.source,args.sink=int(args.source),int(args.sink)
        source_idx,sink_idx=None,None

        topiter=nx.topological_sort(G)
        for i,v in enumerate(topiter):
            if v==args.source:
                source_idx=i
                break
        nodes=[args.source]
        for j,v in enumerate(topiter):
            nodes.append(v)
            if v==args.sink:
                sink_idx=i+j+1
                break
        
        if source_idx==None or sink_idx==None:
            logging.fatal("Unkown source/sink pair: %d,%d"%(args.source,args.sink))

        b=bubbles.Bubble(G,args.source,args.sink,source_idx,sink_idx,nodes)

        nn=max([node for node in G.nodes() if type(node)==int])+1

        bnodes=list(set(b.nodes)-set([b.source,b.sink]))
        sg=G.subgraph(bnodes)

        offsets=dict()
        for sid in G.node[b.source]['offsets']:
            offsets[sid]=G.node[b.source]['offsets'][sid]+len(G.node[b.source]['seq'])

        sourcesamples=set(G.node[b.source]['offsets'].keys())
        sinksamples=set(G.node[b.sink]['offsets'].keys())
        paths=sourcesamples.intersection(sinksamples)

        G.node[b.source]['aligned']=1
        G.node[b.sink]['aligned']=1

        res=refine_bubble(sg,b,offsets,paths, **vars(args))

        if res!=None:
            bubble,ng,path2start,path2end=res
            G,nn=replace_bubble(G,b,ng,path2start,path2end,nn)

    if args.outfile==None:
        fn=args.graph[0].replace(".gfa",".realigned.gfa")
    else:
        fn=args.outfile

    logging.info("Prune and contract nodes...")
    prune_nodes(G)
    contract(G,[n for n in nx.topological_sort(G) if type(n)!=str])
    logging.info("Done.")

    logging.info("Write refined graph to: %s"%fn)
    write_gfa(G,"",outputfile=fn)

def replace_bubble(G,bubble,ng,path2start,path2end,nn):
    assert(nn not in G)

    bubblenodes=bubble.nodes[1:-1] #exclude source and sink node
    
    for node in bubblenodes: #remove all bubblenodes from the original graph
        G.remove_node(node)

    mapping={}
    for node in ng.nodes(): #add nodes from newly aligned graph to original graph
        mapping[node]=nn
        nn+=1
    
    ng=nx.relabel_nodes(ng,mapping) #relabel nodes according to a unique integer range

    for node,data in ng.nodes(data=True): #add nodes from newly aligned graph to original graph
        G.add_node(node,**data)

    for edge in ng.edges(data=True):
        G.add_edge(edge[0],edge[1],**edge[2])

    for sid in path2start:
        startnode=mapping[path2start[sid][0]]
        if G.has_edge(bubble.source,startnode):
            G[bubble.source][startnode]['paths'].add(sid)
        else:
            G.add_edge(bubble.source,startnode,ofrom='+',oto='+',paths=set([sid]))

    for sid in path2end:
        endnode=mapping[path2end[sid][0]]
        if G.has_edge(endnode,bubble.sink):
            G[endnode][bubble.sink]['paths'].add(sid)
        else:
            G.add_edge(endnode,bubble.sink,ofrom='+',oto='+',paths=set([sid]))


    #SHOULD BE ABLE TO SKIP THIS HERE, AS THIS WILL BE DONE OVER THE ENTIRE GRAPH ANYWAY...
    #Just one possible path from source to start, contract nodes
    # if len(G.out_edges(bubble.source))==1 and type(bubble.source)!=str:
    #     # assert(len(set(path2start.values()))==1)
    #     startnode=mapping[path2start.values()[0][0]]
    #     G.node[bubble.source]['seq']+=G.node[startnode]['seq']
    #     for to in G[startnode]:
    #         d=G[startnode][to]
    #         G.add_edge(bubble.source,to,**d)
    #     G.remove_node(startnode)

    #Just one possible path from end to sink, contract nodes
    # if len(G.in_edges(bubble.sink))==1 and type(bubble.sink)!=str:
    #     # assert(len(set(path2end.values()))==1)
    #     endnode=mapping[path2end.values()[0][0]]
    #     G.node[bubble.sink]['seq']=G.node[endnode]['seq']+G.node[bubble.sink]['seq']
    #     G.node[bubble.sink]['offsets']=G.node[endnode]['offsets']
    #     for e0,e1,d in G.in_edges(endnode,data=True):
    #         G.add_edge(e0,bubble.sink,**d)
    #     G.remove_node(endnode)

    return G,nn

def refine_bubble(sg,bubble,offsets,paths,**kwargs):
    source=bubble.source
    sink=bubble.sink

    if len(bubble.nodes)==3:
        logging.fatal("Indel bubble, no point realigning.")
        return

    #TODO: if bubble contains structural variant edge, track these or simply refuse realignment!

    aobjs=[]

    uniqueonly=False

    t0=time.time()
    if kwargs['uniqueonly']:
        d={}
        #extract all paths
        for sid in paths:
            seq=extract(sg,sg.graph['id2path'][sid])
            if len(seq)>0:
                if seq in d:
                    d[seq].append(str(sid))
                else:
                    d[seq]=[str(sid)]

        if len(d)==1:
            logging.debug("Nothing to refine for bubble: %s - %s"%(bubble.source,bubble.sink))
            return

        aobjs=[(",".join(d[seq]),seq) for seq in d]

    else:
        for sid in paths:
            seq=extract(sg,sg.graph['id2path'][sid])
            if len(seq)>0:
                aobjs.append((str(sid),seq))

                # if seq in d:
                #     d[seq].append(str(sid))
                # else:
                #     d[seq]=[str(sid)]

        if len(aobjs)==1:
            logging.debug("Nothing to refine for bubble: %s - %s"%(bubble.source,bubble.sink))
            return
    t1=time.time()
    logging.debug("Extracting sequence for paths: %s through bubble <%s,%s> took: %.4f seconds."%(paths,bubble.source,bubble.sink,t1-t0))


    # logging.info("Realigning bubble (pid=%s) between <%s> and <%s>, %d alleles, with %s (max size %dbp, in nodes=%d)."%(os.getpid(),bubble.source,bubble.sink,len(aobjs),kwargs['method'],bubble.maxsize,len(bubble.nodes)-2))

    # for name,seq in aobjs:
    #     if len(seq)>200:
    #         logging.debug("IN %s: %s...%s (%d bp)"%(name.rjust(4,' '),seq[:100],seq[-100:],len(seq)))
    #     else:
    #         logging.debug("IN %s: %s (%d bp)"%(name.rjust(4,' '),seq,len(seq)))

    if kwargs['method']!="reveal_rem": #use custom multiple sequence aligner to refine bubble structure
        ng=msa2graph(aobjs,
                        msa=kwargs['method'],
                        minconf=kwargs['minconf'],
                        parameters=kwargs['parameters'],
                        constrans=kwargs['constrans'],
                        nrefinements=kwargs['nrefinements'],
                        consgap=kwargs['consgap']
                        )
        if ng==None:
            logging.fatal("MSA using %s for bubble: %s - %s failed."%(kwargs['method'],source,sink))
            return

    else: #use reveal with different settings
        ng,idx=align(aobjs, minlength=kwargs['minlength'],
                            minn=kwargs['minn'],
                            seedsize=kwargs['seedsize'],
                            maxmums=kwargs['maxmums'],
                            wpen=kwargs['wpen'],
                            wscore=kwargs['wscore'],
                            gcmodel=kwargs['gcmodel'],
                            sa64=kwargs['sa64'])
        T=idx.T
        seq2node(ng,T) #transfer sequence to node attributes

    #map edge atts back to original graph
    for n1,n2,data in ng.edges(data=True):
        newpaths=set()
        for p in data['paths']:
            for x in ng.graph['id2path'][p].split(','):
                newpaths.add(int(x))
        data['paths']=newpaths

    #map node atts back to original graph
    for node,data in ng.nodes(data=True):
        newoffsets={}
        for sid in data['offsets']:
            for x in ng.graph['id2path'][sid].split(','):
                newoffsets[int(x)]=data['offsets'][sid]
        data['offsets']=newoffsets

    ng.graph['paths']=sg.graph['paths']
    ng.graph['path2id']=sg.graph['path2id']
    ng.graph['id2path']=sg.graph['id2path']

    mapping={}
    
    path2start=dict()
    path2end=dict()

    #map nodes back to original offsets and idspace, and determine first/last node for every path
    for node,data in ng.nodes(data=True):
        for sid in data['offsets']:
            if sid not in path2start or data['offsets'][sid]<path2start[sid][1]:
                path2start[sid]=(node,data['offsets'][sid])

        for sid in data['offsets']:
            if sid not in path2end or data['offsets'][sid]>path2end[sid][1]:
                path2end[sid]=(node,data['offsets'][sid])

        corrected=dict()
        for sid in data['offsets']:
            corrected[sid]=data['offsets'][sid]+offsets[sid]

        ng.node[node]['offsets']=corrected

    return bubble,ng,path2start,path2end

def align_worker(G,chunk,outputq,kwargs):
    try:
        logging.info("Worker with pid=%d started on subgraph of length: %d"%(os.getpid(),len(G)))

        rchunk=[]
        for b in chunk:

            logging.debug("Start realign bubble (pid=%d) between <%s> and <%s>, max allele size %dbp (in nodes=%d)."%(os.getpid(),b.source,b.sink,b.maxsize,len(b.nodes)-2))

            G.node[b.source]['aligned']=1
            G.node[b.sink]['aligned']=1
            
            bnodes=list(set(b.nodes)-set([b.source,b.sink]))
            sg=G.subgraph(bnodes).copy()
            
            offsets=dict()
            for sid in G.node[b.source]['offsets']:
                offsets[sid]=G.node[b.source]['offsets'][sid]+len(G.node[b.source]['seq'])

            sourcesamples=set(G.node[b.source]['offsets'].keys())
            sinksamples=set(G.node[b.sink]['offsets'].keys())
            paths=sourcesamples.intersection(sinksamples)
            
            t0=time.time()
            rb=refine_bubble(sg,b,offsets,paths,**kwargs)
            t1=time.time()
            logging.debug("Realign bubble (pid=%d) between <%s> and <%s>, max allele size %dbp (in nodes=%d), took %.2f seconds."%(os.getpid(),b.source,b.sink,b.maxsize,len(b.nodes)-2,t1-t0))

            if rb==None:
                continue
            else:
                rchunk.append(rb)
                if len(rchunk)==kwargs['chunksize']:
                    t0=time.time()
                    outputq.put(rchunk)
                    t1=time.time()
                    logging.debug("Added chunk to queue in %.2f seconds."%(t1-t0))
                    rchunk=[]

        if len(rchunk)>0:
            t0=time.time()
            outputq.put(rchunk)
            t1=time.time()
            logging.debug("Added chunk to queue in %.2f seconds."%(t1-t0))

        logging.info("Worker with pid=%d is done."%(os.getpid()))
        outputq.put(-1)
    except Exception,e:
        logging.fatal("Worker with pid=%d failed at bubble <%s,%s>: %s"%(os.getpid(),b.source,b.sink,str(e)))
        exit(1)

def graph_worker(G,nn,outputq,aworkers,totbubbles):
    deadworkers=0
    nworkers=len(aworkers)
    refinedbubbles=0

    while True:

        if deadworkers==nworkers:
            break

        # try:
        t0=time.time()
        # data=outputq.get(timeout=.5)
        data=outputq.get()
        t1=time.time()
        if data!=-1:
            logging.info("Getting chunk of size %d from queue took %.4f seconds."%(len(data),t1-t0))

        # except mp.queues.Empty: #nothing to do, check if all workers are still alive...
        #     logging.info("Empty queue check on workers.")
        #     for wi in range(len(aworkers)):
        #         # p,fn,args=aworkers[wi]
        #         p=aworkers[wi]
        #         if not p.is_alive() and p.exitcode!=0: #one of the workers was killed! maybe oom...
        #             # if retry: #start a new worker and continue processing whatever is left on the queue
        #             #     logging.error("Worker %d died with exitcode: %d, start a new worker!"%(p.pid,p.exitcode))
        #             #     np=mp.Process(target=fn, args=args)
        #             #     np.start()
        #             #     aworkers[wi]=((np,fn,args)) #update it
        #             # else:
        #             raise Exception("Worker %d died with exitcode: %d. Stop refining."%(p.pid,p.exitcode))
        #     continue

        if data==-1: #worker was done
            logging.info("Worker signaled that its done.")
            deadworkers+=1
        else:
            for d in data:
                refinedbubbles+=1
                bubble,ng,path2start,path2end=d
                t0=time.time()
                G,nn=replace_bubble(G,bubble,ng,path2start,path2end,nn)
                t1=time.time()
                logging.info("Replacing bubble (%d/%d): <%s,%s> took %.4f seconds."%(refinedbubbles,totbubbles,bubble.source,bubble.sink,t1-t0))

        for wi in range(len(aworkers)):
            p=aworkers[wi]
            if not p.is_alive() and p.exitcode!=0: #one of the workers was killed! maybe oom...
                raise Exception("Worker %d died with exitcode: %d. Stop refining."%(p.pid,p.exitcode))


def refine_all(G, **kwargs):
    realignbubbles=[]
    
    if kwargs['minsize']==None:
        kwargs['minsize']=kwargs['minlength']

    #detect all bubbles
    logging.info("Extracting bubbles...")

    for b in bubbles.bubbles(G):

        if kwargs['complex']:
            if b.issimple():
                logging.debug("Skipping bubble <%s,%s>, not complex."%(b.source,b.sink))
                continue

        if kwargs['nogaps']:
            spansgap=False
            for n in b.nodes:
                if 'N' in G.nodes[n]['seq']:
                    logging.info("Skipping bubble <%s,%s>, bubble spans a gap."%(b.source,b.sink))
                    spansgap=True
                    break
            if spansgap:
                continue

        if kwargs['simple']:
            if not b.issimple():
                logging.debug("Skipping bubble <%s,%s>, not simple."%(b.source,b.sink))
                continue

        if b.maxsize-b.minsize<kwargs['mindiff']:
            logging.debug("Skipping bubble <%s,%s>, diff between smallest and largest allele (%dbp) is smaller than mindiff=%d."%(b.source,b.sink,b.maxsize-b.minsize,kwargs['mindiff']))
            continue

        if kwargs['maxdiff'] and b.maxsize-b.minsize>kwargs['maxdiff']:
            logging.debug("Skipping bubble <%s,%s>, diff between smallest and largest allele (%dbp) is larger than maxdiff=%d."%(b.source,b.sink,b.maxsize-b.minsize,kwargs['maxdiff']))
            continue

        if b.minsize<kwargs['minsize']:
            logging.debug("Skipping bubble <%s,%s>, smallest allele (%dbp) is smaller than minsize=%d."%(b.source,b.sink,b.minsize,kwargs['minsize']))
            continue

        if b.maxsize>kwargs['maxsize']:
            logging.warn("Skipping bubble <%s,%s>, largest allele (%dbp) is larger than maxsize=%d."%(b.source,b.sink,b.maxsize,kwargs['maxsize']))
            continue

        if kwargs['maxcumsize']!=None:
            if b.cumsize>kwargs['maxcumsize']:
                logging.warn("Skipping bubble <%s,%s>, cumulative size %d is larger than maxcumsize=%d."%(b.source,b.sink,b.cumsize,kwargs['maxcumsize']))
                continue

        if b.cumsize<kwargs['mincumsize']:
            logging.debug("Skipping bubble <%s,%s>, cumulative size %d is smaller than mincumsize=%d."%(b.source,b.sink,b.cumsize,kwargs['mincumsize']))
            continue

        if len(b.nodes)==3:
            logging.debug("Skipping bubble <%s,%s>, indel, no point in realigning."%(b.source,b.sink))
            continue

        b.G=None #remove reference to Graph
        realignbubbles.append(b)
    
    logging.info("Done.")

    if len(realignbubbles)==0:
        logging.info("No bubbles that qualify for realignment.")
    else:
        realignbubbles.sort(key=lambda b: b.source_idx)
        distinctbubbles=[realignbubbles[0]]
        p=0
        i=1
        for i in xrange(i,len(realignbubbles)):
            if realignbubbles[i].source_idx >= realignbubbles[p].sink_idx:
                distinctbubbles.append(realignbubbles[i])
                p=i
            else:
                logging.debug("Skipping realignment for: <%s,%s> - is contained in <%s,%s>"%(realignbubbles[i].source, realignbubbles[i].sink, realignbubbles[p].source, realignbubbles[p].sink))

        logging.info("Realigning a total of %d bubbles"%len(distinctbubbles))
        nn=max([node for node in G.nodes() if type(node)==int])+1

        if kwargs['nproc']>1:
            # inputq = mp.Queue()

            nworkers=kwargs['nproc']-1

            outputq = mp.Queue(nworkers*2)

            aworkers=[]

            shuffle(distinctbubbles) #make sure not all the big telomeric bubbles end up with one worker

            if nworkers>len(distinctbubbles):
                nworkers=len(distinctbubbles)

            # chunksize=50
            # i=0
            # while (i*chunksize)<len(distinctbubbles):
            #     chunk=distinctbubbles[(i*chunksize):((i+1)*chunksize)]
            #     print "Putting chunk:",i,len(chunk)
            #     inputq.put( (i,chunk) ) #,False
            #     i+=1

            chunksize=int(math.floor(len(distinctbubbles)/float(nworkers)))

            for i in range(nworkers):
                logging.info("Starting worker: %d"%i)
                
                if i==nworkers-1:
                    chunk=distinctbubbles[(i*chunksize):]
                else:
                    chunk=distinctbubbles[(i*chunksize):(i*chunksize)+chunksize]

                #create a subgraph for this chunk such that the worker doesnt need to load the entire graph
                t0=time.time()
                Gs=G.subgraph([node for bubble in chunk for node in bubble.nodes]).copy()
                t1=time.time()
                logging.info("Created subgraph in %.2f seconds."%(t1-t0))

                p=mp.Process(target=align_worker, args=(Gs,chunk,outputq,kwargs))
                # p=mp.Process(target=align_worker, args=(G,inputq,outputq,kwargs))
                # p=mp.Process(target=align_worker, args=(G,i,nworkers,distinctbubbles,outputq,kwargs))
                aworkers.append(p) #(p,align_worker,(G,inputq,outputq,kwargs)))

            try:
                for p in aworkers:
                    p.start()

                graph_worker(G,nn,outputq,aworkers,len(distinctbubbles))
                
            except Exception, e:
                logging.fatal("%s"%str(e))
                # for p,fn,args in aworkers:
                for p in aworkers:
                    p.terminate()
                outputq.close()
                exit(1)
            
            outputq.close()
            # inputq.close()

            logging.info("Waiting for workers to finish...")

            # for p,fn,args in aworkers:
            for p in aworkers:
                p.join()

            logging.info("Done.")

        else:
            for bubble in distinctbubbles:
                G.node[bubble.source]['aligned']=1
                G.node[bubble.sink]['aligned']=1

                bnodes=list(set(bubble.nodes)-set([bubble.source,bubble.sink]))
                
                t0=time.time()
                sg=G.subgraph(bnodes).copy()
                t1=time.time()
                logging.info("Extract subgraph for: <%s,%s> took %.4f seconds."%(bubble.source,bubble.sink,t1-t0))
                
                offsets=dict()
                for sid in G.node[bubble.source]['offsets']:
                    offsets[sid]=G.node[bubble.source]['offsets'][sid]+len(G.node[bubble.source]['seq'])

                sourcesamples=set(G.node[bubble.source]['offsets'].keys())
                sinksamples=set(G.node[bubble.sink]['offsets'].keys())
                paths=sourcesamples.intersection(sinksamples)

                # t0=time.time()
                res=refine_bubble(sg,bubble,offsets,paths, **kwargs)
                # t1=time.time()
                # logging.info("Refining bubble: <%s,%s> took %.4f seconds."%(bubble.source,bubble.sink,t1-t0))

                if res==None:
                    continue
                else:
                    bubble,ng,path2start,path2end=res
                    # G,nn=replace_bubble(G,bubble,ng,path2start,path2end,nn)
                    # t0=time.time()
                    G,nn=replace_bubble(G,bubble,ng,path2start,path2end,nn)
                    # t1=time.time()
                    # logging.info("Replacing bubble: <%s,%s> took %.4f seconds."%(bubble.source,bubble.sink,t1-t0))

    return G

def msa2graph(aobjs,idoffset=0,msa='muscle',parameters="",minconf=0,constrans=2,consgap=True,nrefinements=100):

    nn=idoffset
    ng=nx.DiGraph()
    ng.graph['paths']=[]
    ng.graph['path2id']=dict()
    ng.graph['id2path']=dict()
    ng.graph['id2end']=dict()

    maxsize=0
    for name,seq in aobjs:
        sid=len(ng.graph['paths'])
        ng.graph['path2id'][name]=sid
        ng.graph['id2path'][sid]=name
        ng.graph['id2end'][sid]=len(seq)
        ng.graph['paths'].append(name)
        if len(seq)>maxsize:
            maxsize=len(seq)

    uid=uuid.uuid4().hex
    tempfiles=[]

    if msa in {'muscle','pecan','msaprobs','probcons'}:
        logging.debug("Trying to construct MSA with %s, minconf=%d."%(msa,minconf))

        if msa=='muscle':
            cmd="muscle -in %s.fasta -quiet"%uid
            fasta_writer(uid+".fasta",aobjs)
            tempfiles.append("%s.fasta"%uid)
        elif msa=='probcons':
            # cmd="probcons %s.fasta -pre 1 -annot %s.conf"%(uid,uid)
            cmd="probcons %s.fasta -annot %s.conf %s"%(uid,uid,parameters) #-p /Users/jasperlinthorst/Documents/phd/probcons/nw.txt
            fasta_writer(uid+".fasta",aobjs)
            tempfiles.append("%s.fasta"%uid)
            tempfiles.append("%s.conf"%uid)
        elif msa=='pecan':
            cmd="pecan -G %s.fasta -F %s.*.fasta -l -p %s.conf %s && cat %s.fasta"%(uid,uid,uid,parameters,uid)
            for i,(name,seq) in enumerate(aobjs): #pecan wants sequence in separate files
                fasta_writer("%s.%d.fasta"%(uid,i),[(name,seq)])
                tempfiles.append("%s.%d.fasta"%(uid,i))
            tempfiles.append("%s.fasta"%uid)
            tempfiles.append("%s.conf"%uid)
        elif msa=='msaprobs':
            cmd="msaprobs %s.fasta -annot %s.conf %s"%(uid,uid,parameters)
            fasta_writer(uid+".fasta",aobjs)
            tempfiles.append("%s.fasta"%uid)
            tempfiles.append("%s.conf"%uid)
        else:
            logging.fatal("Unkown multiple sequence aligner: %s"%msa)
            sys.exit(1)
        
        seqs=[""]*len(aobjs)
        names=[""]*len(aobjs)

        try:
            DEVNULL = open(os.devnull, 'wb')
            for a in subprocess.check_output([cmd],shell=True,stderr=DEVNULL).split(">")[1:]:
                x=a.find('\n')
                name=a[:x]
                seq=a[x+1:].replace("\n","")
                names[ng.graph['path2id'][name]]=name
                seqs[ng.graph['path2id'][name]]=seq
        except Exception as e:
            logging.fatal("System call to %s failed: \"%s\""%(msa,e.output))
            return

        confidence=[100]*len(seq) #initialize to 100% accuracy for each column

        if os.path.exists("%s.conf"%uid): #if there's an annotation file that accompanies the msa
            with open("%s.conf"%uid) as annot:
                for i,line in enumerate(annot):
                    confidence[i]=float(line.strip()) #expected percentage of correct pairwise matches in the i'th column of the msa...
                    if confidence[i]<1: #consider it a ratio, otherwise a percentage
                        confidence[i]=confidence[i]*100

    else:
        pl=probconslib.probcons()
        t0=time.time()
        aln=pl.align(aobjs,consistency=constrans,refinement=nrefinements,pretraining=0,consgap=consgap)
        t1=time.time()
        logging.debug("ProbCons MSA took %.4f seconds for %d alleles with maxsize=%d."%((t1-t0),len(aobjs),maxsize))

        seqs=[""]*len(aobjs)
        names=[""]*len(aobjs)
        for name,seq in aln[0]:
            names[ng.graph['path2id'][name]]=name
            seqs[ng.graph['path2id'][name]]=seq
        confidence=aln[1]

    for i,seq in enumerate(seqs):
        # if len(seq)>200:
        #     logging.debug("OUT %s: %s...%s"%(str(i).rjust(4, ' '),seq[0:100],seq[-100:]))
        #     logging.debug("CONF    : %s...%s"%("".join([str(c/10) for c in confidence[:100]]),"".join([str(c/10) for c in confidence[-100:]])))
        # else:
        logging.debug("OUT %s: %s"%(str(i).rjust(4, ' '),seq))
    
    logging.debug("CONF    : %s"%"".join([str(c/10) for c in confidence]))
    


    offsets={o:-1 for o in range(len(seqs))}
    nid=nn
    for i in xrange(len(seqs[0])):
        col={}
        base2node={}
        sid2node={}
        p=confidence[i]

        for j in xrange(len(seqs)):
            if seqs[j][i] in col:
                col[seqs[j][i]].add(j)
            else:
                col[seqs[j][i]]=set([j])
            if seqs[j][i]!='-':
                offsets[j]+=1

        for base in col:
            if i==0:
                assert(len(col[base])>0)
                # if len(col[base])>0:
                if p>=minconf:
                    ng.add_node(nid,seq=base,offsets={sid:offsets[sid] for sid in offsets if sid in col[base]},p=[p])
                    base2node[base]=nid
                    for sid in col[base]:
                        assert(sid not in sid2node)
                        sid2node[sid]=nid
                    nid+=1
                else: #new node per sequence
                    for sid in col[base]:
                        ng.add_node(nid,seq=base,offsets={sid:offsets[sid]},p=[p])
                        assert(sid not in sid2node)
                        sid2node[sid]=nid
                        if base in base2node:
                            base2node[base].append(nid)
                        else:
                            base2node[base]=[nid]
                        nid+=1
            else:

                if p>=minconf and pp>=minconf:
                    for pbase in pcol:
                        overlap=pcol[pbase].intersection(col[base])
                        if len(overlap)==0:
                            continue
                        elif len(overlap)==len(col[base])==len(pcol[pbase]): #append seq
                            ng.node[pbase2node[pbase]]['seq']+=base
                            ng.node[pbase2node[pbase]]['p']+=[p]
                            
                            base2node[base]=pbase2node[pbase]
                            
                            for sid in overlap:
                                assert(sid not in sid2node)
                                sid2node[sid]=sid2pnode[sid]
                        else:
                            assert(len(overlap)>0)
                            if base not in base2node: #if not already there
                                ng.add_node(nid,seq=base,offsets=dict(),p=[p])
                                base2node[base]=nid
                                for sid in col[base]:
                                    assert(sid not in sid2node)
                                    sid2node[sid]=nid
                                nid+=1
                            for sid in overlap:
                                ng.node[base2node[base]]['offsets'][sid]=offsets[sid]

                            ng.add_edge(pbase2node[pbase],base2node[base],paths=overlap,oto='+',ofrom='+')

                elif p<minconf and pp>=minconf:

                    for sid in col[base]:
                        ng.add_node(nid,seq=base,offsets={sid:offsets[sid]},p=[p])
                        ng.add_edge(sid2pnode[sid],nid,paths={sid},oto='+',ofrom='+')
                        sid2node[sid]=nid
                        if base in base2node:
                            base2node[base].append(nid)
                        else:
                            base2node[base]=[nid]
                        nid+=1

                elif p>=minconf and pp<minconf:
                    
                    ng.add_node(nid,seq=base,offsets=dict(),p=[p])
                    for sid in col[base]:
                        ng.node[nid]['offsets'][sid]=offsets[sid]
                        if not ng.has_edge(sid2pnode[sid],nid):
                            ng.add_edge(sid2pnode[sid],nid,paths={sid},oto='+',ofrom='+')
                        else:
                            ng[sid2pnode[sid]][nid]['paths'].add(sid)
                        sid2node[sid]=nid
                        base2node[base]=nid
                    nid+=1

                elif p<minconf and pp<minconf:
                    for sid in col[base]:
                        ng.node[sid2pnode[sid]]['seq']+=base
                        ng.node[sid2pnode[sid]]['p'].append(p)
                    sid2node=sid2pnode

                else:
                    logging.error("Impossible combination!")
                    sys.exit(1)

        assert(len(sid2node)==len(seqs))
        sid2pnode=sid2node
        pbase2node=base2node
        pcol=col
        pp=p

    #remove gaps from graph
    remove=[]
    for node,data in ng.nodes(data=True):
        incroffset=False
        if data['seq'][0]=='-':
            incroffset=True

        data['seq']=data['seq'].replace("-","")
        if data['seq']=="":
            remove.append(node)
        elif incroffset:
            for sid in data['offsets']:
                data['offsets'][sid]+=1

        if len(data['offsets'])>1:
            data['aligned']=1
        else:
            data['aligned']=0

    for node in remove:
        ine=ng.in_edges(node,data=True)
        oute=ng.out_edges(node,data=True)
        for in1,in2,ind in ine:
            for out1,out2,outd in oute:
                overlap=ind['paths'].intersection(outd['paths'])
                if len(overlap)>=1:
                    if ng.has_edge(in1,out2):
                        ng[in1][out2]['paths']=ng[in1][out2]['paths'] | overlap
                    else:
                        ng.add_edge(in1,out2,paths=overlap,ofrom='+',oto='+')

    ng.remove_nodes_from(remove)

    #contract edges
    updated=True
    while updated:
        updated=False
        for v,t in ng.edges():
            if ng.out_degree(v)==ng.in_degree(t)==1:
                if ng.node[v]['offsets'].keys()==ng.node[t]['offsets'].keys():
                    ng.node[v]['seq']+=ng.node[t]['seq']
                    for suc in ng.successors(t):
                        ng.add_edge(v,suc,**ng[t][suc])
                    ng.remove_node(t)
                    updated=True
                    break

    for tmpfile in tempfiles:
        try:
            os.remove(tmpfile)
        except Exception as e:
            logging.fatal("Failed to remove tmp file: \"%s\""%tmpfile)
            return

    # logging.debug("%d nodes in refined graph:",ng.number_of_nodes())
    # for node in ng:
        # logging.debug("%s: %s"%(node,ng.node[node]['seq']))

    return ng


import reveallib
import reveallib64
from utils import *
from multiprocessing.pool import Pool
import signal
import os
import math
import argparse
import logging
import intervaltree
import matplotlib
import sortedcontainers
import time

def plot(plt,anchors,sep,wait=True,nc='r',rc='g',color=None,edges=False,lines=False,alpha=1,args=None):
    
    if len(anchors)==0:
        return

    if len(anchors[0])==2: #unaligned blocks
        for start,stop in anchors:
            ax = plt.axes()
            if start<sep: #ref
                ax.add_patch(
                        matplotlib.patches.Rectangle(
                            (start, 0), #bottom left
                            stop-start, #width
                            sep, #height #should be qry length!
                            alpha=.25,
                            color="blue"
                        )
                    )
            else:
                ax.add_patch(
                        matplotlib.patches.Rectangle(
                            (0, start-sep), #bottom left
                            sep, #width
                            stop-start, #height
                            alpha=.25,
                            color="grey"
                        )
                    )
    elif len(anchors[0])==3: #mums
        for l,sps,revcomp in anchors:
            if revcomp:
                plt.plot( (sps[0],sps[0]+l), ((sps[1]-sep)+l, (sps[1]-sep)),'%s-'%rc,alpha=alpha)
            else:
                plt.plot( (sps[0],sps[0]+l), ((sps[1]-sep), (sps[1]-sep)+l),'%s-'%nc,alpha=alpha)
    elif len(anchors[0])==4: #synteny blocks, without orientation
        for anchor in anchors:
            s1,e1,s2,e2=anchor
            ax = plt.axes()
            ax.add_patch(
                    matplotlib.patches.Rectangle(
                        (s1, s2-sep), #bottom left
                        e1-s1, #width
                        e2-s2, #height
                        alpha=.5,
                        color=color
                    )
                )
    elif len(anchors[0])==5: #synteny blocks with orientation
        for anchor in anchors:
            s1,e1,s2,e2,revcomp=anchor
            ax = plt.axes()
            ax.add_patch(
                    matplotlib.patches.Rectangle(
                        (s1, s2-sep), #bottom left
                        e1-s1, #width
                        e2-s2, #height
                        alpha=.25,
                        color="green" if revcomp else "red"
                    )
                )
    elif len(anchors[0])==8: #synteny blocks with score and ctg
        
        if edges:
            for c in [0,2]:
                anchors.sort(key=lambda a:a[c])

                xedges,yedges=[],[]
                
                panchor=None
                for anchor in anchors:

                    s1,e1,s2,e2,revcomp,score,ref,ctg=anchor
                    
                    if panchor!=None:
                        ps1,pe1,ps2,pe2,prevcomp,pscore,pref,pctg=panchor

                        if pctg!=ctg and pref!=ref:
                            panchor=anchor
                            continue

                        if c==0:
                            xedges.append(pe1)
                            xedges.append(s1)
                            xedges.append(None)
                            
                            if prevcomp:
                                yedges.append(ps2-sep)
                            else:
                                yedges.append(pe2-sep)
                            
                            if revcomp:
                                yedges.append(e2-sep)
                            else:
                                yedges.append(s2-sep)
                            yedges.append(None)

                        else:

                            if prevcomp:
                                xedges.append(ps1)
                            else:
                                xedges.append(pe1)

                            if revcomp:
                                xedges.append(e1)
                            else:
                                xedges.append(s1)

                            xedges.append(None)
                            
                            yedges.append(pe2-sep)
                            yedges.append(s2-sep)
                            yedges.append(None)

                    panchor=anchor

                if c==0:
                    plt.plot(xedges,yedges,'b--',alpha=alpha)
                else:
                    plt.plot(xedges,yedges,'y--',alpha=alpha)

        if lines:
            rcxpoints,xpoints=[],[]
            rcypoints,ypoints=[],[]

            for anchor in anchors:
                s1,e1,s2,e2,revcomp,score,ref,ctg=anchor

                # plt.text(s1+((e1-s1)/2),(s2-sep)+(((e2-sep)-(s2-sep))/2) ,str(anchor),fontsize=6)
                
                if revcomp:
                    # plt.plot((s1,e1), (e2-sep,s2-sep),'g-')
                    rcxpoints.append(s1)
                    rcxpoints.append(e1)
                    rcxpoints.append(None)
                    rcypoints.append(e2-sep)
                    rcypoints.append(s2-sep)
                    rcypoints.append(None)
                else:
                    # plt.plot((s1,e1), (s2-sep,e2-sep),'r-')
                    xpoints.append(s1)
                    xpoints.append(e1)
                    xpoints.append(None)
                    ypoints.append(s2-sep)
                    ypoints.append(e2-sep)
                    ypoints.append(None)
            
            plt.plot(xpoints,ypoints,'r-' if color==None else '%s-'%color,alpha=alpha)
            plt.plot(rcxpoints,rcypoints,'g-' if color==None else '%s-'%color,alpha=alpha)

        else: #plot squares
            for anchor in anchors:
                s1,e1,s2,e2,revcomp,score,ref,ctg=anchor
                ax = plt.axes()
                ax.add_patch(
                        matplotlib.patches.Rectangle(
                            (s1, s2-sep), #bottom left
                            e1-s1, #width
                            e2-s2, #height
                            alpha=.25,
                            color="green" if revcomp else "red"
                        )
                    )

    if wait:
        plt.show()
    else:
        plt.draw()

def addctginfo(mums,ctg2range):
    logging.debug("Augment contig information.")
    #add ref information to mums
    mums.sort(key=lambda m: m[1][0]) #sort mums by ref domain
    intvidx=0
    for i in range(len(mums)):
        while mums[i][1][0]>ctg2range[intvidx][1]:
            intvidx+=1
        mums[i]=mums[i]+(intvidx,)
    
    #add contig information to mums
    mums.sort(key=lambda m: m[1][1]) #sort mums by query domain
    intvidx=0
    for i in range(len(mums)):
        while mums[i][1][1]>ctg2range[intvidx][1]:
            intvidx+=1
        mums[i]=mums[i]+(intvidx,)
    logging.debug("Done.")
    return mums

def transform_cmd(args):
    for qry in args.contigs:
        logging.info("Running transform for %s"%qry)
        transform(args,qry)
        logging.info("Done")

def transform(args,qry):

    if not args.interactive:
        matplotlib.use("agg")

    if args.plot:
        from matplotlib import pyplot as plt

    if args.output==None:
        prefix=os.path.splitext(os.path.basename(qry))[0]
    else:
        prefix=args.output
    
    refnames=[]
    ctgnames=[]

    if args.sa64:
        idx=reveallib64.index()
    else:
        idx=reveallib.index()

    ctg2range=[]
    for sample in [args.reference[0],qry]:
        idx.addsample(os.path.basename(sample))

        for name,seq in fasta_reader(sample, cutN=args.cutn):
            if len(seq)<args.minctglength:
                logging.debug("Skip transform for contig: %s"%name)
                continue

            intv=idx.addsequence(seq)
            ctg2range.append(intv)

            if sample==args.reference[0]:
                refnames.append(name)
            else:
                ctgnames.append(name)
    T=idx.T

    logging.info("Compute mums.")
    idx.construct(rc=False)
    mums=addctginfo(idx.getmums(args.minlength),ctg2range)
    logging.info("Done, %d mums."%len(mums))
    
    if args.cluster:
        logging.info("Cluster mums by diagonals.")
        blocks=clustermumsbydiagonal(mums,maxdist=args.maxdist,minclustsize=args.mincluster,rcmums=False)
        logging.info("Done, %d clusters."%len(blocks))
    else:
        blocks=[(mum[1][0], mum[1][0]+mum[0], mum[1][1], mum[1][1]+mum[0], mum[2], mum[0], mum[3], mum[4]) for mum in mums]
    
    # rcidx=idx.copy()
    # rcidx.construct(rc=True)
    # mums+=rcidx.getmums(args.minlength)

    logging.info("Compute RC mums.")
    idx.construct(rc=True)
    rcmums=addctginfo(idx.getmums(args.minlength),ctg2range)
    logging.info("Done, %d rc mums."%len(rcmums))

    sep=idx.nsep[0]
    idxn=idx.n

    rlength=idx.nsep[0]
    qlength=idxn-idx.nsep[0]

    del idx

    if args.cluster:
        logging.info("Cluster rc mums by anti-diagonals.")
        if len(rcmums)==0:
            rcblocks = [(mum[1][0], mum[1][0] + mum[0], mum[1][1], mum[1][1] + mum[0], mum[2], mum[0], mum[3], mum[4]) for mum in rcmums]
        else:
            rcblocks=clustermumsbydiagonal(rcmums,maxdist=args.maxdist,minclustsize=args.mincluster,rcmums=True)
            logging.info("Done, %d rc clusters."%len(rcblocks))
    else:
        rcblocks=[(mum[1][0], mum[1][0]+mum[0], mum[1][1], mum[1][1]+mum[0], mum[2], mum[0], mum[3], mum[4]) for mum in rcmums]
    
    blocks+=rcblocks

    if args.plot:
        plot(plt,blocks,sep,wait=False,lines=True,alpha=0.2,args=args)

    # if args.plot:
    #     plot(blocks,sep,wait=False,lines=True)

    logging.info("Start glocal chaining for filtering anchors (reference).")
    
    # blocks.sort(key=lambda b: b[1]-b[0])
    # logging.info("Largest ref block: %s"%str(blocks[-1]))
    # minbacktrack=blocks[-1][1]-blocks[-1][0]

    nbefore=len(blocks)
    syntenyblocks=blocks
    nafter=None
    refiteration=0
    # maxiter=1
    while nbefore!=nafter:# or refiteration==maxiter:
        logging.info("Glocal chain iteration %d"%refiteration)
        nbefore=len(syntenyblocks)
        syntenyblocks=glocalchain(syntenyblocks,rlength,qlength,ctg2range,rearrangecost=args.rearrangecost,
                                                            inversioncost=args.inversioncost,
                                                            _lambda=args._lambda,
                                                            eps=args.eps,
                                                            useheap=args.useheap, 
                                                            lastn=args.lastn,
                                                            lastbp=args.lastbp,
                                                            alfa=args.alfa,
                                                            gapopen=args.gapopen,
                                                            axis=0)
        nafter=len(syntenyblocks)
        logging.info("Anchor before chaining: %s"%nbefore)
        logging.info("Anchor after chaining: %s"%nafter)
        refiteration+=1
        
        if args.plot:
            plot(plt,syntenyblocks,sep,wait=False,lines=True,color='k',alpha=.7)
    
    logging.info("%d anchors remain after glocal chaining (reference)."%len(syntenyblocks))

    logging.info("Start glocal chaining for filtering anchors (query).")

    nbefore=len(syntenyblocks)
    nafter=None
    qryiteration=0
    while nbefore!=nafter:# or qryiteration==maxiter:
        logging.info("Glocal chain iteration %d"%qryiteration)
        nbefore=len(syntenyblocks)
        syntenyblocks=glocalchain(syntenyblocks,rlength,qlength,ctg2range,rearrangecost=args.rearrangecost,
                                                                inversioncost=args.inversioncost,
                                                                _lambda=args._lambda,
                                                                eps=args.eps,
                                                                useheap=args.useheap, 
                                                                lastn=args.lastn,
                                                                lastbp=args.lastbp,
                                                                alfa=args.alfa,
                                                                gapopen=args.gapopen,
                                                                axis=1)
        nafter=len(syntenyblocks)
        logging.info("Anchor before chaining: %s"%nbefore)
        logging.info("Anchor after chaining: %s"%nafter)
        qryiteration+=1

    # G=localcolinearchains(syntenyblocks,rlength,qlength,rearrangecost=rearrangecost,inversioncost=inversioncost)
    # chain,rcchain=colinearchains(syntenyblocks,rlength,qlength)
    logging.info("%d anchors remain after glocal chaining (query)."%len(syntenyblocks))

    if args.plot:
        plot(plt,syntenyblocks,sep,wait=False,lines=True,color='b',alpha=.7)

    #take the intersection of both the chains
    # logging.info("Determine intersection between the chains...")
    # syntenyblocks=list(set(rsyntenyblocks) & set(qsyntenyblocks))
    # logging.info("Done. %d chains remain."%len(qsyntenyblocks))

    # logging.info("Remove anchors that are contained in other clusters."
    # syntenyblocks=remove_contained_blocks(blocks)
    # logging.info("Done, %d anchors remain."%len(syntenyblocks))
    # logging.info("Done.")

    
    logging.info("Merge consecutive blocks.")
    syntenyblocks=merge_consecutive(syntenyblocks)
    logging.info("%d blocks after merging consecutive blocks."%len(syntenyblocks))

    # if args.plot:
    #     plot(syntenyblocks,sep,wait=True,lines=True,color='b')

    # logging.info("Merge consecutive blocks.")
    # syntenyblocks=merge_consecutive(syntenyblocks)
    # logging.info("%d blocks after merging consecutive blocks."%len(syntenyblocks))

    if args.greedy:
        logging.info("Assign overlap between MUMs in a greedy manner.")
        syntenyblocks=remove_overlap_greedy_blocks(syntenyblocks)
        logging.info("Done.")
    else:
        logging.info("Assign overlap between MUMs in a conservative manner.")
        syntenyblocks=remove_overlap_conservative_blocks(syntenyblocks)
        logging.info("Done.")
    
    logging.info("Remove all blocks that are shorter than minchainsum (%d)."%args.minchainsum)
    syntenyblocks=[b for b in syntenyblocks if b[5] >= args.minchainsum]
    logging.info("%d blocks after filtering for minchainsum."%len(syntenyblocks))

    logging.info("Merge consecutive blocks.")
    syntenyblocks=merge_consecutive(syntenyblocks)
    logging.info("%d blocks after merging consecutive blocks."%len(syntenyblocks))

    if args.optimise and len(syntenyblocks)>1:

        weight,cost,edgecosts=chainscore(syntenyblocks, rlength, qlength, ctg2range,rearrangecost=args.rearrangecost,inversioncost=args.inversioncost,_lambda=args._lambda,eps=args.eps,alfa=args.alfa,gapopen=args.gapopen) #determine the actual cost of the glocal chain 
        score=weight-cost

        assert(len(edgecosts) == len(syntenyblocks)+1)

        iteration=0
        
        while True:
            iteration+=1
            logging.info("Optimise chain, iteration %d."%iteration)
            tsyntenyblocks,tweight,tcost,tedgecosts=optimise(syntenyblocks,rlength, qlength, ctg2range,rearrangecost=args.rearrangecost,inversioncost=args.inversioncost,_lambda=args._lambda,eps=args.eps,alfa=args.alfa,gapopen=args.gapopen)
            nscore=tweight-tcost
            
            if nscore<=score:
                break
            else:
                score=nscore
                syntenyblocks=tsyntenyblocks
                weight=tweight
                cost=tcost
                edgecosts=tedgecosts
                syntenyblocks=merge_consecutive(syntenyblocks)

        logging.info("Done. %d blocks after optimisation."%len(syntenyblocks))

    syntenyblocks=merge_consecutive(syntenyblocks)
    weight,cost,edgecosts=chainscore(syntenyblocks, rlength, qlength, ctg2range,rearrangecost=args.rearrangecost,inversioncost=args.inversioncost,_lambda=args._lambda,eps=args.eps,alfa=args.alfa,gapopen=args.gapopen) #determine the actual cost of the glocal chain 
    score=weight-cost

    assert(len(edgecosts) == len(syntenyblocks)+1)

    if args.outputbed: #before extending to the edges of the contig, output the breakpoint regions

        logging.info("Write bedfile with contig mappings on reference to: %s.bed"%prefix)
        with open(prefix+".bed",'w') as bedout:

            block2ctgidx=dict()
            pctgid=None

            ctgid2lastblock=dict()
            ci=0

            syntenyblocks.sort(key=lambda b: b[2]) #sort by query
            for i,block in enumerate(syntenyblocks): #sorted by query
                s1,e1,s2,e2,o,score,refid,ctgid=block
                if ctgid!=pctgid:
                    if pctgid!=None:
                        ctgid2lastblock[pctgid]=ci
                    ci=0
                else:
                    ci+=1
                block2ctgidx[block]=ci
                pctgid=ctgid

            ctgid2lastblock[pctgid]=ci

            syntenyblocks.sort(key=lambda b: b[0]) #sort by reference
            bedout.write("#reference\trefbegin\trefend\tcontig:segmentidx:lastsegmentidx:begin:end\tscore:cost\torientation\taln-start\taln-end\n")

            pblock=None

            for i,block in enumerate(syntenyblocks): #sorted by reference
                s1,e1,s2,e2,o,score,refid,ctgid=block
                
                if i>0:
                    ps1,pe1,ps2,pe2,po,pscore,prefid,pctgid=pblock
                else:
                    pblock=None

                cost=edgecosts[i] #cost to connect to pblock to block

                if i<len(syntenyblocks)-2:
                    nblock=syntenyblocks[i+1]
                    ns1,ne1,ns2,ne2,no,nscore,nrefid,nctgid=nblock
                else:
                    nblock=None

                ctgoffsets=ctg2range[ctgid]
                refoffsets=ctg2range[refid]

                if pblock!=None and prefid==refid:
                    start=(s1-refoffsets[0])-((s1-pe1)/2)
                else:
                    start=s1-refoffsets[0]

                if nblock!=None and nrefid==refid:
                    end=(e1-refoffsets[0])+((ns1-e1)/2)
                else:
                    end=e1-refoffsets[0]

                qstart=s2-ctgoffsets[0]
                qend=e2-ctgoffsets[0]

                chromname=refnames[refid].split()[0]

                qi=block2ctgidx[block]
                bedout.write("%s\t%d\t%d\t%s:%d:%d:%d:%d\t%d:%d\t%s\t%d\t%d\n"%(chromname, #chrom
                                                                start, #start
                                                                end, #end
                                                                ctgnames[ctgid-len(refnames)].split()[0], #name, make sure there's no whitespace to comply with bed 'format'
                                                                qi,
                                                                ctgid2lastblock[ctgid],
                                                                qstart,
                                                                qend,
                                                                score,
                                                                cost,
                                                                '+' if o==False else '-', #strand
                                                                s1-refoffsets[0], #thick start
                                                                e1-refoffsets[0]) #thick end
                                                                #itemRgb
                                                                #blockCount
                                                                #blockSizes
                                                                #blockStarts
                                                            )

                #bedout.write("%s\t%d\t%d\t%s\t%s\t%s\t%s\n"%(refnames[refid], pe1-refoffsets[0], s1-refoffsets[0], ctgnames[ctgid-len(refnames)], ctgnames[pctgid-len(refnames)], 'n' if po==False else 'r', 'n' if o==False else 'r'))

                pblock=block

    if args.plot:
        plot(plt,syntenyblocks,sep,wait=False,args=args)

    logging.debug("Extend %d blocks to query borders."%len(syntenyblocks))
    extendblocks(syntenyblocks,ctg2range)
    logging.debug("Done.")

    if args.plot:
        for start,end in ctg2range:
            if start<sep:
                plt.axvline(x=start, ymin=0, ymax=idxn-sep, linewidth=.1, linestyle='solid')
            else:
                plt.axhline(y=start-sep, xmin=0, xmax=sep, linewidth=.1, linestyle='solid')

        plot(plt,syntenyblocks,sep,wait=False,edges=False,args=args)
        plt.xlim(0,rlength)
        plt.ylim(0,qlength)

        if args.interactive:
            plt.show()
        else:
            plt.savefig("%s.png"%(prefix))

        plt.clf()

    #determine the subset of mappable contigs from ref and qry
    mappablectgs=set()
    for s1,e1,s2,e2,o,score,refid,ctgid in syntenyblocks:
        mappablectgs.add(ctgid)
        mappablectgs.add(refid)

    if len(mappablectgs)!=0:
        logging.info("Write breakpoint graph to: %s.gfa"%prefix)
        write_breakpointgraph(syntenyblocks,T,refnames,ctgnames,mappablectgs,prefix)
    else:
        logging.info("No mappable contigs.")

def clustermumsbydiagonal(mums,maxdist=90,minclustsize=65,rcmums=True):
    logging.debug("Sorting anchors by diagonals...")
    if rcmums:
        mums.sort(key=lambda m: (m[1][0]+(m[1][1]+m[0]), m[1][0]-(m[1][1]+m[0])) ) #sort mums by anti-diagonal, then diagonal
    else:
        mums.sort(key=lambda m: (m[1][0]-m[1][1], m[1][0]+m[1][1])) #sort mums by diagonal, then anti-diagonal
    logging.debug("Done.")

    l,sps,rc,ctg,ref=mums[0]
    clusters=[(sps[0],sps[0]+l,sps[1],sps[1]+l,rc,l,ctg,ref)]

    update_progress(0,len(mums))
    for i in xrange(1,len(mums)):
        update_progress(i,len(mums))

        l,sps,rc,ctg,ref=mums[i]
        s1,e1,s2,e2,prc,score,pctg,pref=clusters[-1]

        if rcmums:
            d=mums[i][1][0]+(mums[i][1][1]+mums[i][0])
            pd=e1+s2
        else:
            d=mums[i][1][0]-mums[i][1][1]
            pd=s1-s2

        if d==pd and pctg==ctg and pref==ref: #same diagonal and same contigs
            dist=mums[i][1][0]-e1
            assert(dist>=0)
            if dist < maxdist:
                if rc==0:
                    clusters[-1]=(s1,sps[0]+l,s2,sps[1]+l,rc,score+l,ctg,ref)
                else:
                    clusters[-1]=(s1,sps[0]+l,sps[1],e2,rc,score+l,ctg,ref)
            else:
                clusters.append((sps[0],sps[0]+l,sps[1],sps[1]+l,rc,l,ctg,ref))
        else:
            clusters.append((sps[0],sps[0]+l,sps[1],sps[1]+l,rc,l,ctg,ref))

    return [c for c in clusters if c[5]>=minclustsize]


def write_breakpointgraph(syntenyblocks,T,refnames,ctgnames,mappablectgs,outputprefix):
    #build a breakpoint graph, that we can write to GFA
    G=nx.MultiDiGraph()
    start=uuid.uuid4().hex
    end=uuid.uuid4().hex
    G.graph['startnodes']=[start]
    G.graph['endnodes']=[end]
    G.graph['paths']=[]
    G.graph['path2id']={}
    G.graph['id2path']={}

    G.add_node(start,offsets=dict())
    G.add_node(end,offsets=dict())
    
    pid=0
    for name in refnames:
        if pid in mappablectgs:
            # name=os.path.splitext(os.path.basename(reference))[0]+"_"+name
            name=os.path.basename(outputprefix+"_"+name)
            G.graph['paths'].append(name)
            G.graph['path2id'][name]=pid
            G.graph['id2path'][pid]=name
            G.node[start]['offsets'][pid]=0
        else:
            logging.info("No contigs were mapped to: %s"%name)
        pid+=1

    for name in ctgnames:
        if pid in mappablectgs:
            name="*"+name #prefix so we can recognise the two paths afterwards
            G.graph['paths'].append(name)
            G.graph['path2id'][name]=pid
            G.graph['id2path'][pid]=name
            G.node[start]['offsets'][pid]=0
        else:
            logging.info("Contig: %s could not be uniquely placed on the reference"%name)
        pid+=1

    #write the reference layout of the query sequences
    syntenyblocks.sort(key=lambda b: b[0]) #TODO: check if not already the case..
    prefid=None
    pnid=None
    l=0

    mapping=dict()
    nid=0    

    for i,block in enumerate(syntenyblocks):
        s1,e1,s2,e2,o,score,refid,ctgid=block

        mapping[(s2,e2)]=nid

        if refid!=prefid:
            if prefid!=None:
                G.add_edge(pnid,end,paths=set([prefid]),ofrom="+", oto="+")
            pnid=start
            l=0

        if o==0:
            G.add_node(nid,seq=T[s2:e2],offsets={refid:l})
        else:
            G.add_node(nid,seq=rc(T[s2:e2]),offsets={refid:l})
        
        G.add_edge(pnid,nid,paths=set([refid]),ofrom="+", oto="+")
        prefid=refid
        pnid=nid
        nid+=1
        l+=e2-s2
        
        if i!=len(syntenyblocks)-1: #add gap node, so we later know which bubbles are caused by gaps in the assembly
            gapsize=1 #TODO: if specified use reference to add a gap
            G.add_node(nid,seq="N"*gapsize,offsets={refid:l})
            l+=gapsize
            G.add_edge(pnid,nid,paths=set([refid]),ofrom="+", oto="+")
            pnid=nid
            nid+=1

    G.add_edge(pnid,end,paths=set([refid]),ofrom="+", oto="+")

    writeorg=True
    if writeorg: #write the original layout of the query sequences, so we can reconstruct the input afterwards
        syntenyblocks.sort(key=lambda b: b[2])
        pctgid=None
        pnid=None
        
        l=0
        for nid,block in enumerate(syntenyblocks):
            s1,e1,s2,e2,o,score,refid,ctgid=block
            nid=mapping[(s2,e2)]
            
            if ctgid!=pctgid:
                if pctgid!=None:
                    G.add_edge(pnid,end,paths=set([pctgid]),ofrom="+" if o==0 else "-", oto="+")
                pnid=start
                l=0
                po=0

            G.node[nid]['offsets'][ctgid]=l
            
            l+=e2-s2
            G.add_edge(pnid,nid,paths=set([ctgid]),ofrom="+" if po==0 else "-", oto="+" if o==0 else "-")

            po=o
            pctgid=ctgid
            pnid=nid

        G.add_edge(pnid,end,paths=set([ctgid]),ofrom="+" if o==0 else "-", oto="+")

    write_gfa(G,None,outputfile=outputprefix if outputprefix.endswith(".gfa") else outputprefix+".gfa")

def merge_consecutive(syntenyblocks):
    if len(syntenyblocks)<2:
        return syntenyblocks
    #first merge consecutive blocks in the chain
    syntenyblocks.sort(key=lambda s: s[0]) #order by ref position
    qryorder = sorted(xrange(len(syntenyblocks)), key= lambda i: syntenyblocks[i][2]) #qry order
    qryorder_inv = sorted(xrange(len(syntenyblocks)), key=qryorder.__getitem__) #inverse qry order
    head=0
    
    for ri in xrange(1,len(syntenyblocks)):
        pblock=syntenyblocks[ri-1]
        block=syntenyblocks[ri]
        pqi=qryorder_inv[ri-1] #index within the qryorder of pblock
        qi=qryorder_inv[ri] #index within the qryorder of block
        ps1,pe2,ps2,pe2,po,pscore,prefid,pctgid=pblock #previous block on reference
        s1,e1,s2,e2,o,score,refid,ctgid=block
        es1,ee1,es2,ee2,eo,escore,erefid,ectgid=syntenyblocks[head]
        if ctgid==pctgid:
            if pqi+1==qi and o==po==0:
                syntenyblocks[head]=(es1,e1,es2,e2,eo,escore+score,erefid,ectgid)
            elif pqi-1==qi and o==po==1:
                syntenyblocks[head]=(es1,e1,s2,ee2,eo,escore+score,erefid,ectgid)
            else:
                head+=1
                syntenyblocks[head]=block
        else:
            head+=1
            syntenyblocks[head]=block
    
    while head!=ri:#len(syntenyblocks)-1:
        syntenyblocks.pop()
        head+=1

    return syntenyblocks

def extendblocks(syntenyblocks,ctg2range):

    syntenyblocks.sort(key=lambda s: s[0]) #order by reference position
    
    for i in xrange(len(syntenyblocks)):
        s1,e1,s2,e2,o,score,ref,ctg=syntenyblocks[i]
        
        if i==0: #first
            s1=ctg2range[ref][0]
        else:
            ps1,pe1,ps2,pe2,po,pscore,pref,pctg=syntenyblocks[i-1]
            if pref==ref:
                s1=pe1
            else:
                s1=ctg2range[ref][0]
        
        if i==len(syntenyblocks)-1: #last
            e1=ctg2range[ref][1]
        else:
            ns1,ne1,ns2,ne2,no,nscore,nref,nctg=syntenyblocks[i+1]
            if nref==ref:
                e1+=((ns1-e1)/2)
            else:
                e1=ctg2range[ref][1]

        assert(s1<e1)
        syntenyblocks[i]=(s1,e1,s2,e2,o,score,ref,ctg)

    syntenyblocks.sort(key=lambda s: s[2]) #order by qry position

    for i in xrange(len(syntenyblocks)):
        s1,e1,s2,e2,o,score,ref,ctg=syntenyblocks[i]
        if i==0: #first
            s2=ctg2range[ctg][0]
        else:
            ps1,pe1,ps2,pe2,po,pscore,pref,pctg=syntenyblocks[i-1]
            if pctg==ctg:
                s2=pe2
            else:
                s2=ctg2range[ctg][0]

        if i==len(syntenyblocks)-1: #last
            e2=ctg2range[ctg][1]
        else:
            ns1,ne1,ns2,ne2,no,nscore,nref,nctg=syntenyblocks[i+1]
            if nctg==ctg:
                e2+=((ns2-e2)/2)
            else:
                e2=ctg2range[ctg][1]
        
        assert(s2<e2)
        syntenyblocks[i]=(s1,e1,s2,e2,o,score,ref,ctg)

def optimise(syntenyblocks,rlength, qlength, ctg2range,rearrangecost=1000,inversioncost=1,_lambda=5,eps=1,alfa=1,gapopen=10):

    orgchain=sorted(syntenyblocks,key=lambda c: c[5])
    maxchain=syntenyblocks
    maxchain_weight,maxchain_cost,maxchain_edgecosts=chainscore(maxchain, rlength, qlength, ctg2range, rearrangecost=rearrangecost,inversioncost=inversioncost,_lambda=_lambda,eps=eps,alfa=alfa,gapopen=gapopen)
    maxchainscore=maxchain_weight-maxchain_cost

    stack=[]
    loglevel=logging.getLogger().getEffectiveLevel()

    if loglevel>logging.DEBUG:
        update_progress(0,len(orgchain))
    
    for i in xrange(len(orgchain)):
        if loglevel>logging.DEBUG:
            update_progress(i,len(orgchain))

        tmp=list(stack+orgchain[i+1:])
        weight,cost,edgecosts=chainscore(tmp, rlength, qlength, ctg2range, rearrangecost=rearrangecost,inversioncost=inversioncost,_lambda=_lambda,eps=eps,alfa=alfa,gapopen=gapopen)
        tmpchainscore=weight-cost

        if tmpchainscore<maxchainscore:
            stack.append(orgchain[i]) #keep it
        else:
            logging.debug("Dropped block %s, gain: %d"%(orgchain[i],tmpchainscore-maxchainscore))
            maxchainscore=tmpchainscore
            maxchain=tmp
            maxchain_cost=cost
            maxchain_weight=weight
            maxchain_edgecosts=edgecosts

    logging.debug("Optimal chain has %d blocks and scores: %d"%(len(maxchain),maxchainscore))

    return maxchain,maxchain_weight,maxchain_cost,maxchain_edgecosts

def chainscore(chain, rlength, qlength, ctg2range, rearrangecost=1000, inversioncost=1, _lambda=5, eps=1, alfa=1, gapopen=10):
    
    # logging.debug("rearrangecost=%d, inversioncost=%d, _lambda=%d, eps=%d, alfa=%d, gapopen=%d"%(rearrangecost, inversioncost, _lambda, eps, alfa, gapopen))

    if len(chain)==0:
        start=(0,0,rlength,rlength,0,0,0,0)
        end=(rlength,rlength,rlength+qlength,rlength+qlength,0,0,0,0)
        cost=gapcost(start,end,rearrangecost=rearrangecost,inversioncost=inversioncost,_lambda=_lambda,eps=eps,gapopen=gapopen,axis=0)
        return 0,cost,[cost]

    chain.sort(key=lambda s: s[0]) #order by reference position
    qryorder = sorted(xrange(len(chain)), key= lambda i: chain[i][2]) #qry order
    qryorder_inv = sorted(xrange(len(chain)), key=qryorder.__getitem__) #inverse qry order

    lastqstart,lastqend=ctg2range[chain[-1][7]]
    if chain[0][4]==0:
        end=(rlength,rlength,lastqend,lastqend,chain[0][4])
    else:
        end=(rlength,rlength,lastqstart,lastqstart,chain[0][4])

    firstqstart,firstqend=ctg2range[chain[0][7]]
    if chain[0][4]==0:
        start=(0,0,firstqstart,firstqstart,chain[0][4])
    else:
        start=(0,0,firstqend,firstqend,chain[0][4])
    
    #count out of order traversals
    rearrangements=0
    inversions=0

    startcost=gapcost(start,chain[0],rearrangecost=rearrangecost,inversioncost=inversioncost,_lambda=_lambda,eps=eps,gapopen=gapopen,axis=0)

    cost=startcost
    edgecosts=[startcost]

    weight=alfa*chain[0][5]
    
    for ri in xrange(1,len(chain)):

        pblock=chain[ri-1]
        block=chain[ri]

        ps1,pe1,ps2,pe2,po,pscore,pref,pctg=pblock
        s1,e1,s2,e2,o,score,ref2,ctg=block
        weight+=(alfa*score)

        # xgap=0#s1-pe1
        
        pqi=qryorder_inv[ri-1] #index within the qryorder of pblock
        qi=qryorder_inv[ri] #index within the qryorder of block

        if pctg==ctg and pref==ref2:

            if (pqi==qi-1) or (pqi==qi+1): #check if the two blocks are colinear
                gc=gapcost(pblock,block,rearrangecost=rearrangecost,inversioncost=inversioncost,_lambda=_lambda,eps=eps,gapopen=gapopen,axis=0)
                cost+=gc
                edgecosts.append(gc)
            else: #all other options use rearrangement penalty
                rearrangements+=1
                cost+=(gapopen+rearrangecost)
                edgecosts.append(gapopen+rearrangecost)
        
        else: #cross contigs
            
            if o==0:
                if qi>0:
                    pqs1,pqe1,pqs2,pqe2,pqo,pqscore,pq_ref,pq_ctg=chain[qryorder[qi-1]]
                else:
                    pq_ctg='start'
            else:
                if qi<len(qryorder)-1:
                    pqs1,pqe1,pqs2,pqe2,pqo,pqscore,pq_ref,pq_ctg=chain[qryorder[qi+1]]
                else:
                    pq_ctg='end'

            if po==0:
                if pqi<len(qryorder)-1:
                    nqs1,nqe1,nqs2,nqe2,nqo,nqscore,nq_ref,nq_ctg=chain[qryorder[pqi+1]]
                else:
                    nq_ctg='end'
            else:
                if pqi>0:
                    nqs1,nqe1,nqs2,nqe2,nqo,nqscore,nq_ref,nq_ctg=chain[qryorder[pqi-1]]
                else:
                    nq_ctg='start'

            if pq_ctg==ctg or nq_ctg==pctg: #there exists another block on this query contig before changing contigs, so has to be rearranged
                rearrangements+=1
                cost+=(gapopen+rearrangecost)
                edgecosts.append((gapopen+rearrangecost))
            else:
                edgecosts.append(gapopen) #simple traversal between two contigs

    endcost=gapcost(chain[-1],end,rearrangecost=rearrangecost,inversioncost=inversioncost,_lambda=_lambda,eps=eps,gapopen=gapopen,axis=0)

    cost+=endcost

    edgecosts.append(endcost)

    return weight,cost,edgecosts

def update_progress(i,n):
    fullbar=100
    if (i+1) % (n/fullbar if n>fullbar else 1)==0 or i+1==n:
        done=int(fullbar*((i+1)/float(n)))
        todo=fullbar-done
        sys.stdout.write('\r[%s%s]'%("#"*done," "*todo))
        if i+1==n:
            sys.stdout.write('\n')
        sys.stdout.flush()

def glocalchain(syntenyblocks, rlength, qlength, ctg2range, rearrangecost=1000, inversioncost=1, lastn=50, lastbp=10000, useheap=False, axis=0, _lambda=5, eps=1, alfa=1, gapopen=10):

    sep=rlength
    
    #add some dummy blocks for the contig start/ends
    if axis==0:
        for refid,(refstart,refend) in enumerate(ctg2range):
            if refstart>=sep:
                break
            if refid==0:
                start=(refstart,refstart,None,None,0,0,None,None)
            syntenyblocks.append((refend,refend,None,None,0,0,None,None))
        end=syntenyblocks[-1]

    if axis==1:
        first=True
        for ctgid,(ctgstart,ctgend) in enumerate(ctg2range):
            if ctgstart<sep:
                continue
            if first:
                start=(None,None,ctgstart,ctgstart,0,0,None,None)
                first=False
            syntenyblocks.append((None,None,ctgend,ctgend,0,0,None,None))
        end=syntenyblocks[-1]

    if axis==0: #sort by ref
        c1,c2=0,2
    else: #sort by qry
        c1,c2=2,0

    syntenyblocks.sort(key=lambda s: (s[c1],-s[5]) ) #order by reference position, then score

    if useheap:
        heap=sortedcontainers.SortedList()
        heap.add((0,start))
    else:
        heap=[(0,start)]+[None]*(len(syntenyblocks))

    G={b:None for b in syntenyblocks}

    maxscore=None

    n=len(syntenyblocks)
    
    bt=range(n+1)

    update_progress(0,n)

    pri=0
    t0=time.time()

    deepest=0
    # best=None

    for ri in xrange(n):
        block=syntenyblocks[ri]

        while syntenyblocks[deepest][c1+1]<block[c1]:
            deepest+=1

        if ri%1000==0:
            t1=time.time()
            sec=t1-t0
            bd=ri-pri
            logging.debug("Blocks per sec: %d"%(bd/sec))
            t0=t1
            pri=ri

        update_progress(ri,n)

        s1,e1,s2,e2,o,score,refid,ctgid=block

        trace=False
        # starttrace=105637436
        # endtrace=starttrace+10
        # if s1>=starttrace and s1<endtrace: # and refid==ctgtrace:
        # #     # if block==(4499237, 4502780, 9008394, 9011937, 0, 3543, 0, 1) or block==end:
        #     logging.info("BLOCK: %s"%str(block))
        #     print "deepest",syntenyblocks[deepest], syntenyblocks[deepest][c1+1]
        #     trace=True

        bestscore=None
        bestblock=None
        bestcost=0
        # checkedbest=False

        l=0
        
        for j in bt: #back track on the heap

            if useheap:
                if j>=len(heap):
                    break
                cscore,pblock=heap[-j]
            else:
                i=(ri+1)-j-1
                if i<0:
                    break
                cscore,pblock=heap[i]
            
            # if best==None or cscore==best:
                # checkedbest=True

            ps1,pe1,ps2,pe2,po,pscore,prefid,pctgid=pblock

            if (pblock[c1]==block[c1] and prefid!=None and refid!=None) or (pblock[c1+1]>=block[c1+1] and prefid!=None and refid!=None):
                continue

            if (pblock[c2]>=block[c2] and prefid!=None and refid!=None) and (pblock[c2+1]<=block[c2+1] and prefid!=None and refid!=None):
                continue            

            l+=1

            if bestscore!=None:
                if cscore<=bestscore:
                    if useheap:
                        break
                    else:
                        if block[c1]-pblock[c1]>lastbp and l>=lastn and pblock[c1]<syntenyblocks[deepest][c1]:
                            break
                        else:
                            continue

            #if block is a dummy block, make it relative to pblock, if possible
            if block[6]==None and pblock[6]!=None: #update current block to be relative to pblock
                if axis==0:
                    _block=(s1, e1, pe2 if po==0 else ps2, pe2 if po==0 else ps2, po, 0, prefid, pctgid)
                else:
                    _block=(pe1 if po==0 else ps1, pe1 if po==0 else ps1, s2, e2, po, 0, prefid, pctgid)
            else:
                _block=block

            #if pblock is a dummy block, make it relative to block, if possible
            if pblock[6]==None and block[6]!=None:
                if axis==0:
                    _pblock=(ps1, pe1, s2 if o==0 else e2, s2 if o==0 else e2, o, 0, refid, ctgid)
                else:
                    _pblock=(s1 if o==0 else e1, s1 if o==0 else e1, ps2, pe2, o, 0, refid, ctgid)
            else:
                _pblock=pblock

            #if blocks come from same query contig and reference contig, compute gapcost, else introduce rearrangement cost
            if _pblock[6]==_block[6]!=None and _pblock[7]==_block[7]!=None:
                c=gapcost(_pblock,_block,rearrangecost=rearrangecost,inversioncost=inversioncost,eps=eps,_lambda=_lambda,gapopen=gapopen,axis=axis)
            elif _pblock[6]==_block[6]==None and _pblock[7]==_block[7]==None: #connect two dummy blocks
                c=gapopen+(abs(block[c1]-(pblock[c1+1]))*eps)
            else: #blocks cross contigs or ref without passing a dummy block, introduce rearrangement cost

                pblockctgstart,pblockctgend=ctg2range[_pblock[7]]
                blockctgstart,blockctgend=ctg2range[_block[7]]
                pblockrefstart,pblockrefend=ctg2range[_pblock[6]]
                blockrefstart,blockrefend=ctg2range[_block[6]]

                if _pblock[6]==_block[6] and axis==0:
                    if _pblock[4]==0:
                        cp=abs( pblockctgend-_pblock[3])
                    else:
                        cp=abs( _pblock[2]-pblockctgstart)

                    if _block[4]==0:
                        cb=abs( blockctgend-_block[3] )
                    else:
                        cb=abs( _block[2]-blockctgstart )

                    c=gapopen+min((rearrangecost,((cp+cb)*eps)))

                elif _pblock[7]==_block[7] and axis==1:

                    if _pblock[4]==0:
                        cp=abs( pblockrefend-_pblock[1])
                    else:
                        cp=abs( _pblock[0]-pblockrefstart)

                    if _block[4]==0:
                        cb=abs( _block[0]-blockrefstart )
                    else:
                        cb=abs( blockrefend-_block[1])

                    c=gapopen+min((rearrangecost,((cp+cb)*eps)))
                else:
                    c=rearrangecost+gapopen+(abs(block[c1]-(pblock[c1+1]))*eps)

            assert(c>=0)

            if trace:
                logging.info("Connect to PBLOCK: %s costs %s, depth=%s, lastbp=%d, cscore,%s, cscore-c=%d, bestscore=%s"%(pblock,c,l,block[c1]-pblock[c1],cscore,cscore-c,bestscore))

            if bestscore==None or cscore-c > bestscore:
                bestscore=cscore-c
                bestblock=pblock
                bestcost=c

            if not useheap:
                if block[c1]-pblock[c1]>lastbp and l>=lastn and pblock[c1]<syntenyblocks[deepest][c1]:
                    break

        # if l>lastn:
            # logging.info("Forced deeper %d backtrack for block: %s"%(l,block))

        cscore=bestscore+(alfa*score)

        # if best==None or cscore>best:
            # best=cscore

        if useheap:
            heap.add((cscore,block))
        else:
            heap[ri+1]=(cscore,block)
        
        if maxscore==None or maxscore<cscore:
            maxscore=cscore
            maxnode=block

        if trace:
            logging.info("CONNECT TO BLOCK: %s, score=%s, cost=%s, depth=%s"%(bestblock,bestscore,bestcost,l))

        G[block]=(bestblock,bestscore)

    node,cscore=G[end]

    chain=[]
    while node!=start:# and node!=startrc:
        if node[6]!=None: #only add to the chain if it is an actual anchor, exclude contig endpoints
            chain.append(node)
        s1,e1,s2,e2,o,score,refid,ctgid=node
        nnode,score=G[node]
        if node==nnode:
            logging.fatal("Loop in chain!")
            sys.exit(1)
        node=nnode

    logging.info("Optimal glocal chain contains: %d anchors and scores %d"%(len(chain),cscore))

    return chain[::-1]

def gapcost(block1,block2,rearrangecost=10000,inversioncost=0,eps=0,_lambda=0.5,gapopen=10,axis=0):

    if axis==0: #sorted by ref
        c1,c2=0,2
    else: #sorted by qry
        c1,c2=2,0

    assert(block1[c1]<=block2[c1])

    d1=block2[c1]-block1[c1+1]

    if block1[4]==block2[4]==0: #both normal orientation
        if block2[c2]<block1[c2]:#always has to be rearranged!
            indelcost=rearrangecost
            substitutioncost=eps*(d1 if d1>0 else 0) #do not penalize if overlap
            return gapopen+indelcost+substitutioncost
        else:
            d2=block2[c2]-block1[c2+1]
            indelcost=min((rearrangecost,_lambda*abs(d1-d2)))
            substitutioncost=eps*max(((d1 if d1<d2 else d2),0))
            return gapopen+indelcost+substitutioncost

    elif block1[4]==block2[4]==1: #both reverse comp orientation
        if block2[c2]>block1[c2]: #always has to be rearranged!
            indelcost=rearrangecost
            substitutioncost=eps*(d1 if d1>0 else 0)
            return gapopen+indelcost+substitutioncost
        else:
            d2=block1[c2]-block2[c2+1]
            indelcost=min((rearrangecost,_lambda*abs(d1-d2)))
            substitutioncost=eps*max(((d1 if d1<d2 else d2),0))
            return gapopen+indelcost+substitutioncost

    elif block1[4]==1 and block2[4]==0:
        d1=max((0,d1))
        if block2[c2]>block1[c2]:
            d2=block2[c2]-block1[c2+1]
            d2=max((0,d2))
            indelcost=min((rearrangecost,_lambda*abs(d1-d2)))
            substitutioncost=eps*max(((d1 if d1<d2 else d2),0))
            return gapopen+indelcost+substitutioncost+inversioncost
        else:
            d2=block1[c2]-block2[c2+1]
            d2=max((0,d2))
            indelcost=min((rearrangecost,_lambda*abs(d1-d2)))
            substitutioncost=eps*max(((d1 if d1<d2 else d2),0))
            return gapopen+indelcost+substitutioncost+inversioncost

    else:
        # assert(block1[4]==0 and block2[4]==1)
        d1=max((0,d1))
        if block2[c2]>block1[c2]:
            d2=block2[c2]-block1[c2+1]
            d2=max((0,d2))
            indelcost=min((rearrangecost,_lambda*abs(d1-d2)))
            substitutioncost=eps*max(((d1 if d1<d2 else d2),0))
            return gapopen+indelcost+substitutioncost+inversioncost
        else:
            d2=block1[c2]-block2[c2+1]
            d2=max((0,d2))
            indelcost=min((rearrangecost,_lambda*abs(d1-d2)))
            substitutioncost=eps*max(((d1 if d1<d2 else d2),0))
            return gapopen+indelcost+substitutioncost+inversioncost

def _gapcost(block1,block2,rearrangecost=10000,inversioncost=0,eps=0,_lambda=0.5,gapopen=10,axis=0):

    if axis==0: #sorted by ref
        c1,c2=0,2
    else: #sorted by qry
        c1,c2=2,0

    assert(block1[c1]<=block2[c1])
    
    d1=abs(block2[c1]-block1[c1+1])
    
    if block1[4]==block2[4]==0: #both normal orientation
        if block2[c2]<block1[c2]:#always has to be rearranged!
            indelcost=rearrangecost
            substitutioncost=eps*d1
            return gapopen+indelcost+substitutioncost
        else:
            d2=abs(block2[c2]-block1[c2+1])
            indelcost=min((rearrangecost,_lambda*abs(d1-d2)))
            substitutioncost=eps*(d1 if d1<d2 else d2)
            return gapopen+indelcost+substitutioncost

    elif block1[4]==block2[4]==1: #both reverse comp orientation
        if block2[c2]>block1[c2]: #always has to be rearranged!
            indelcost=rearrangecost
            substitutioncost=eps*d1
            return gapopen+indelcost+substitutioncost
        else:
            d2=abs(block1[c2]-block2[c2+1])
            indelcost=min((rearrangecost,_lambda*abs(d1-d2)))
            substitutioncost=eps*(d1 if d1<d2 else d2)
            return gapopen+indelcost+substitutioncost

    elif block1[4]==1 and block2[4]==0:
        if block2[c2]>block1[c2]:
            d2=abs(block2[c2]-block1[c2+1])
            indelcost=min((rearrangecost,_lambda*abs(d1-d2)))
            substitutioncost=eps*(d1 if d1<d2 else d2)
            return gapopen+indelcost+substitutioncost+inversioncost
        else:
            d2=abs(block1[c2]-block2[c2+1])
            indelcost=min((rearrangecost,_lambda*abs(d1-d2)))
            substitutioncost=eps*(d1 if d1<d2 else d2)
            return gapopen+indelcost+substitutioncost+inversioncost

    else:
        # assert(block1[4]==0 and block2[4]==1)
        if block2[c2]>block1[c2]:
            d2=abs(block2[c2]-block1[c2+1])
            indelcost=min((rearrangecost,_lambda*abs(d1-d2)))
            substitutioncost=eps*(d1 if d1<d2 else d2)
            return gapopen+indelcost+substitutioncost+inversioncost
        else:
            d2=abs(block1[c2]-block2[c2+1])
            indelcost=min((rearrangecost,_lambda*abs(d1-d2)))
            substitutioncost=eps*(d1 if d1<d2 else d2)
            return gapopen+indelcost+substitutioncost+inversioncost


def printSA(index,maxline=100,start=0,end=None,fn="sa.txt"):
    sa=index.SA
    lcp=index.LCP
    t=index.T
    #so=index.SO
    if end==None:
        end=len(sa)
    
    # with open(fn,'w') as f:
    sys.stdout.write("%d\t%d\n"%(len(sa), len(lcp)))
    assert(len(sa)==len(lcp))
    for i in xrange(len(sa)):
        s=sa[i]
        lcpi=lcp[i]

        if i>0 and i<len(sa)-1:
            l1=lcp[i]
            l2=lcp[i+1]
        elif i==len(sa)-1:
            l1=max([lcp[i-1],lcp[i]])
            l2=0
        else:
            l1=0
            l2=lcp[i+1]

        if i>=start and i<=end:
            #f.write("%s\t%s\t%s\n"%(str(s).zfill(8), str(lcpi).zfill(6), t[s:s+maxline].ljust(maxline) if l1<=maxline else t[s:s+maxline]+"..."+t[s+l1-40:s+l1].ljust(maxline) ) )
            sys.stdout.write("%s\t%s\t%s\t%s\t%s\n"%(str(s).zfill(8), str(lcpi).zfill(6), t[s:s+maxline] ,t[s+l1-maxline:s+l1], t[s+l2-maxline:s+l2] ) )

def remove_overlap_conservative_blocks(anchors):
    
    for coord in [0,2]:

        if len(anchors)<=1: #by definition no containment
            return anchors

        anchors.sort(key=lambda m: (m[coord], (m[coord+1]-m[coord])*-1)) #sort by start position, then -1*size
        
        _anchors=[anchors[0]]
        last=anchors[0]
        for anchor in anchors[1:]:
            if anchor[coord] < last[coord+1]: #overlap
                if anchor[coord+1]<=last[coord+1]: #contained
                    continue
            _anchors.append(anchor)
            last=anchor
        anchors=_anchors

        _anchors=[anchors[0]]
        for anchor in anchors[1:]:
            s1,e1,s2,e2,o,score,refid,ctgid=anchor
            ps1,pe1,ps2,pe2,po,pscore,prefid,pctgid=_anchors[-1]

            overlap=(_anchors[-1][coord+1]) - anchor[coord]
            pl=pe1-ps1

            if overlap > 0: #overlap
                
                if score<=overlap:
                    continue

                assert(score-overlap >= 0)

                if o==0:
                    anchor=(s1+overlap,e1,s2+overlap,e2,o,score-overlap if overlap<score else 0,refid,ctgid)
                else:
                    if coord==0:
                        anchor=(s1+overlap,e1,s2,e2-overlap,o,score-overlap if overlap<score else 0,refid,ctgid)
                    else:
                        anchor=(s1,e1-overlap,s2+overlap,e2,o,score-overlap if overlap<score else 0,refid,ctgid)

                assert(anchor[coord+1]>_anchors[-1][coord+1])

                while pl<=overlap or pscore<=overlap:
                    _anchors.pop()
                    ps1,pe1,ps2,pe2,po,pscore,prefid,pctgid=_anchors[-1]
                    overlap=(_anchors[-1][coord+1]) - anchor[coord]
                    if overlap<0:
                        break
                    pl=pe1-ps1

                if overlap>0:                    
                    assert(pscore-overlap >= 0)
                    if po==0:
                        _anchors[-1]=(ps1,pe1-overlap,ps2,pe2-overlap,po,pscore-overlap if overlap<pscore else 0,prefid,pctgid)
                    else:
                        if coord==0:
                            _anchors[-1]=(ps1,pe1-overlap, ps2+overlap,pe2, po,pscore-overlap if overlap<pscore else 0, prefid,pctgid)
                        else:
                            _anchors[-1]=(ps1+overlap,pe1,ps2,pe2-overlap,po,pscore-overlap if overlap<pscore else 0, prefid,pctgid)
            
            _anchors.append(anchor)

        anchors=_anchors

    return anchors

def remove_overlap_greedy_blocks(anchors):
    
    #TODO: remove duplicates!

    for coord in [0,2]:
        if len(anchors)<=1: #by definition no containment
            return anchors
        
        update_progress(0,len(anchors))

        anchors.sort(key=lambda m: (m[coord], (m[coord+1]-m[coord])*-1)) #sort by start position, then -1*size
        
        _anchors=[anchors[0]]
        last=anchors[0]
        for anchor in anchors[1:]:
            if anchor[coord] < last[coord+1]: #overlap
                if anchor[coord+1]<=last[coord+1]: #contained
                    continue
            _anchors.append(anchor)
            last=anchor
        anchors=_anchors

        _anchors=[anchors[0]]
        # for anchor in anchors[1:]:

        for i in xrange(1,len(anchors)):
            anchor=anchors[i]

            update_progress(i,len(anchors))

            s1,e1,s2,e2,o,score,refid,ctgid=anchor
            ps1,pe1,ps2,pe2,po,pscore,prefid,pctgid=_anchors[-1]
            pl=pe1-ps1

            overlap=(_anchors[-1][coord+1]) - anchor[coord]

            if overlap > 0: #overlap

                if pscore > score: #update current anchor

                    if score<=overlap:
                        continue

                    assert(score-overlap >= 0)

                    if o==0:
                        anchor=(s1+overlap,e1,s2+overlap,e2,o,score-overlap if overlap<score else 0,refid,ctgid)
                    else:
                        if coord==0:
                            anchor=(s1+overlap,e1,s2,e2-overlap,o,score-overlap if overlap<score else 0,refid,ctgid)
                        else:
                            anchor=(s1,e1-overlap,s2+overlap,e2,o,score-overlap if overlap<score else 0,refid,ctgid)

                    _anchors.append(anchor)
                else:

                    while pl<=overlap or pscore<=overlap:
                        _anchors.pop()
                        ps1,pe1,ps2,pe2,po,pscore,prefid,pctgid=_anchors[-1]
                        overlap=(_anchors[-1][coord+1]) - anchor[coord]
                        if overlap<0:
                            break
                        pl=pe1-ps1

                    if overlap>0:
                        
                        assert(pl>overlap)
                        assert(pscore>overlap)

                        assert(pscore-overlap >= 0)

                        if po==0:
                            _anchors[-1]=(ps1,pe1-overlap,ps2,pe2-overlap,po,pscore-overlap if overlap<pscore else 0,prefid,pctgid)
                        else:
                            if coord==0:
                                _anchors[-1]=(ps1,pe1-overlap, ps2+overlap,pe2, po,pscore-overlap if overlap<pscore else 0,prefid,pctgid)
                            else:
                                _anchors[-1]=(ps1+overlap,pe1,ps2,pe2-overlap,po,pscore-overlap if overlap<pscore else 0,prefid,pctgid)                    
                    _anchors.append(anchor)
            else:
                _anchors.append(anchor)

        anchors=_anchors
    return anchors

def remove_contained_blocks(anchors):
    #remove duplicates!

    for coord in [0,2]:
        logging.info("Remove overlap in %s dimension."%("first" if coord==0 else "second"))

        if len(anchors)<=1: #by definition no containment
            return anchors

        anchors.sort(key=lambda m: (m[coord], (m[coord+1]-m[coord])*-1) ) #sort by start position, then -1*size
        
        _anchors=[anchors[0]]
        last=anchors[0]

        update_progress(0,len(anchors))

        # for anchor in anchors[1:]:
        for i in xrange(1,len(anchors)):
            anchor=anchors[i]
            update_progress(i,len(anchors))

            if anchor[coord] < last[coord+1]: #overlap
                if anchor[coord+1]<=last[coord+1]: #contained
                    continue
            _anchors.append(anchor)
            last=anchor
        anchors=_anchors

    return anchors

#unused
def remove_overlap_greedy_mums(anchors):
    
    #remove duplicates!
    n=2

    for coord in range(n):
        if len(anchors)<=1: #by definition no containment
            return anchors

        anchors.sort(key=lambda m: (m[1][coord], m[0]*-1)) #sort by start position, then -1*size
        
        _anchors=[anchors[0]]
        last=anchors[0]
        for anchor in anchors[1:]:
            if anchor[1][coord] < last[1][coord]+last[0]: #overlap
                if anchor[1][coord]+anchor[0]<=last[1][coord]+last[0]: #contained
                    continue
            _anchors.append(anchor)
            last=anchor
        anchors=_anchors

        _anchors=[anchors[0]]
        for anchor in anchors[1:]:
            overlap=(_anchors[-1][1][coord]+_anchors[-1][0]) - anchor[1][coord]

            if overlap > 0: #overlap

                if _anchors[-1][0] > anchor[0]:
                    if anchor[2]==0:
                        anchor=(anchor[0]-overlap, (anchor[1][0]+overlap, anchor[1][1]+overlap), anchor[2])
                    else:
                        if coord==0:
                            anchor=(anchor[0]-overlap, (anchor[1][0]+overlap, anchor[1][1]), anchor[2])
                        else:
                            anchor=(anchor[0]-overlap, (anchor[1][0], anchor[1][1]+overlap), anchor[2])

                    _anchors.append(anchor)
                else:

                    while _anchors[-1][0]<=overlap and overlap>0:
                        _anchors.pop()
                        overlap=(_anchors[-1][1][coord]+_anchors[-1][0]) - anchor[1][coord]

                    if overlap>0:
                    
                        if _anchors[-1][2]==0:
                            _anchors[-1]=(_anchors[-1][0]-overlap,_anchors[-1][1],_anchors[-1][2]) #update stack
                        else:
                            if coord==0:
                                _anchors[-1]=(_anchors[-1][0]-overlap,_anchors[-1][1],_anchors[-1][2])
                            else:
                                _anchors[-1]=(_anchors[-1][0]-overlap,(_anchors[-1][1][0]+overlap, _anchors[-1][1][1]),_anchors[-1][2])
                    
                    _anchors.append(anchor)
            else:
                _anchors.append(anchor)

        anchors=_anchors
    return anchors

#unused
def remove_contained_mums(anchors):
    #remove duplicates!

    for coord in range(2):
        if len(anchors)<=1: #by definition no containment
            return anchors

        anchors.sort(key=lambda m: (m[1][coord], m[0]*-1)) #sort by start position, then -1*size
        
        _anchors=[anchors[0]]
        last=anchors[0]
        for anchor in anchors[1:]:
            if anchor[1][coord] < last[1][coord]+last[0]: #overlap
                if anchor[1][coord]+anchor[0]<=last[1][coord]+last[0]: #contained
                    continue
            _anchors.append(anchor)
            last=anchor
        anchors=_anchors

    return anchors

#unused
def remove_overlap_conservative_mums(anchors):
    
    #remove duplicates!
    n=2

    for coord in range(n):
        if len(anchors)<=1: #by definition no containment
            return anchors

        anchors.sort(key=lambda m: (m[1][coord], m[0]*-1)) #sort by start position, then -1*size
        
        _anchors=[anchors[0]]
        last=anchors[0]
        for anchor in anchors[1:]:
            if anchor[1][coord] < last[1][coord]+last[0]: #overlap
                if anchor[1][coord]+anchor[0]<=last[1][coord]+last[0]: #contained
                    continue
            _anchors.append(anchor)
            last=anchor
        anchors=_anchors

        _anchors=[anchors[0]]
        last=anchors[0]
        for anchor in anchors[1:]:
            if anchor[1][coord] < last[1][coord]+last[0]: #overlap
                
                assert(anchor[1][coord]+anchor[0] > last[1][coord]+last[0]) #may not be contained, as we filtered these out already
                
                overlap=(last[1][coord]+last[0])-anchor[1][coord]

                assert(overlap>=0)

                assert(anchor[0]>overlap)
                
                if anchor[2]==0:
                    anchor=(anchor[0]-overlap, (anchor[1][0]+overlap, anchor[1][1]+overlap), anchor[2])
                else:
                    if coord==0:
                        anchor=(anchor[0]-overlap, (anchor[1][0]+overlap, anchor[1][1]), anchor[2])
                    else:
                        anchor=(anchor[0]-overlap, (anchor[1][0], anchor[1][1]+overlap), anchor[2])

                # assert(last[0]>overlap)

                if last[2]==0:
                    _anchors[-1]=(last[0]-overlap,last[1],last[2]) #update last
                else:
                    if coord==0:
                        _anchors[-1]=(last[0]-overlap,(last[1][0], last[1][1]+overlap),last[2])
                    else:
                        _anchors[-1]=(last[0]-overlap,(last[1][0]+overlap, last[1][1]),last[2])

            if _anchors[-1][0]<=0:
                _anchors[-1]=anchor
            else:
                _anchors.append(anchor)

            last=anchor

        anchors=_anchors

    return anchors
